# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter
import mock

from openstack.baremetal.v1 import _common
from openstack.baremetal.v1 import node
from openstack import exceptions
from openstack.tests.unit import base

# NOTE: Sample data from api-ref doc
FAKE = {
    "chassis_uuid": "1",  # NOTE: missed in api-ref sample
    "clean_step": {},
    "console_enabled": False,
    "created_at": "2016-08-18T22:28:48.643434+00:00",
    "driver": "agent_ipmitool",
    "driver_info": {
        "ipmi_password": "******",
        "ipmi_username": "ADMIN"
    },
    "driver_internal_info": {},
    "extra": {},
    "inspection_finished_at": None,
    "inspection_started_at": None,
    "instance_info": {},
    "instance_uuid": None,
    "last_error": None,
    "links": [
        {
            "href": "http://127.0.0.1:6385/v1/nodes/<NODE_ID>",
            "rel": "self"
        },
        {
            "href": "http://127.0.0.1:6385/nodes/<NODE_ID>",
            "rel": "bookmark"
        }
    ],
    "maintenance": False,
    "maintenance_reason": None,
    "name": "test_node",
    "network_interface": "flat",
    "portgroups": [
        {
            "href": "http://127.0.0.1:6385/v1/nodes/<NODE_ID>/portgroups",
            "rel": "self"
        },
        {
            "href": "http://127.0.0.1:6385/nodes/<NODE_ID>/portgroups",
            "rel": "bookmark"
        }
    ],
    "ports": [
        {
            "href": "http://127.0.0.1:6385/v1/nodes/<NODE_ID>/ports",
            "rel": "self"
        },
        {
            "href": "http://127.0.0.1:6385/nodes/<NODE_ID>/ports",
            "rel": "bookmark"
        }
    ],
    "power_state": None,
    "properties": {},
    "provision_state": "enroll",
    "provision_updated_at": None,
    "raid_config": {},
    "reservation": None,
    "resource_class": None,
    "states": [
        {
            "href": "http://127.0.0.1:6385/v1/nodes/<NODE_ID>/states",
            "rel": "self"
        },
        {
            "href": "http://127.0.0.1:6385/nodes/<NODE_ID>/states",
            "rel": "bookmark"
        }
    ],
    "target_power_state": None,
    "target_provision_state": None,
    "target_raid_config": {},
    "updated_at": None,
    "uuid": "6d85703a-565d-469a-96ce-30b6de53079d"
}


class TestNode(base.TestCase):

    def test_basic(self):
        sot = node.Node()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('nodes', sot.resources_key)
        self.assertEqual('/nodes', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)

    def test_instantiate(self):
        sot = node.Node(**FAKE)

        self.assertEqual(FAKE['uuid'], sot.id)
        self.assertEqual(FAKE['name'], sot.name)

        self.assertEqual(FAKE['chassis_uuid'], sot.chassis_id)
        self.assertEqual(FAKE['clean_step'], sot.clean_step)
        self.assertEqual(FAKE['created_at'], sot.created_at)
        self.assertEqual(FAKE['driver'], sot.driver)
        self.assertEqual(FAKE['driver_info'], sot.driver_info)
        self.assertEqual(FAKE['driver_internal_info'],
                         sot.driver_internal_info)
        self.assertEqual(FAKE['extra'], sot.extra)
        self.assertEqual(FAKE['instance_info'], sot.instance_info)
        self.assertEqual(FAKE['instance_uuid'], sot.instance_id)
        self.assertEqual(FAKE['console_enabled'], sot.is_console_enabled)
        self.assertEqual(FAKE['maintenance'], sot.is_maintenance)
        self.assertEqual(FAKE['last_error'], sot.last_error)
        self.assertEqual(FAKE['links'], sot.links)
        self.assertEqual(FAKE['maintenance_reason'], sot.maintenance_reason)
        self.assertEqual(FAKE['name'], sot.name)
        self.assertEqual(FAKE['network_interface'], sot.network_interface)
        self.assertEqual(FAKE['ports'], sot.ports)
        self.assertEqual(FAKE['portgroups'], sot.port_groups)
        self.assertEqual(FAKE['power_state'], sot.power_state)
        self.assertEqual(FAKE['properties'], sot.properties)
        self.assertEqual(FAKE['provision_state'], sot.provision_state)
        self.assertEqual(FAKE['raid_config'], sot.raid_config)
        self.assertEqual(FAKE['reservation'], sot.reservation)
        self.assertEqual(FAKE['resource_class'], sot.resource_class)
        self.assertEqual(FAKE['states'], sot.states)
        self.assertEqual(FAKE['target_provision_state'],
                         sot.target_provision_state)
        self.assertEqual(FAKE['target_power_state'], sot.target_power_state)
        self.assertEqual(FAKE['target_raid_config'], sot.target_raid_config)
        self.assertEqual(FAKE['updated_at'], sot.updated_at)

    def test_normalize_provision_state(self):
        attrs = dict(FAKE, provision_state=None)
        sot = node.Node(**attrs)
        self.assertEqual('available', sot.provision_state)


@mock.patch('time.sleep', lambda _t: None)
@mock.patch.object(node.Node, 'fetch', autospec=True)
class TestNodeWaitForProvisionState(base.TestCase):
    def setUp(self):
        super(TestNodeWaitForProvisionState, self).setUp()
        self.node = node.Node(**FAKE)
        self.session = mock.Mock()

    def test_success(self, mock_fetch):
        def _get_side_effect(_self, session):
            self.node.provision_state = 'manageable'
            self.assertIs(session, self.session)

        mock_fetch.side_effect = _get_side_effect

        node = self.node.wait_for_provision_state(self.session, 'manageable')
        self.assertIs(node, self.node)

    def test_failure(self, mock_fetch):
        def _get_side_effect(_self, session):
            self.node.provision_state = 'deploy failed'
            self.assertIs(session, self.session)

        mock_fetch.side_effect = _get_side_effect

        self.assertRaisesRegex(exceptions.ResourceFailure,
                               'failure state "deploy failed"',
                               self.node.wait_for_provision_state,
                               self.session, 'manageable')

    def test_failure_error(self, mock_fetch):
        def _get_side_effect(_self, session):
            self.node.provision_state = 'error'
            self.assertIs(session, self.session)

        mock_fetch.side_effect = _get_side_effect

        self.assertRaisesRegex(exceptions.ResourceFailure,
                               'failure state "error"',
                               self.node.wait_for_provision_state,
                               self.session, 'manageable')

    def test_enroll_as_failure(self, mock_fetch):
        def _get_side_effect(_self, session):
            self.node.provision_state = 'enroll'
            self.node.last_error = 'power failure'
            self.assertIs(session, self.session)

        mock_fetch.side_effect = _get_side_effect

        self.assertRaisesRegex(exceptions.ResourceFailure,
                               'failed to verify management credentials',
                               self.node.wait_for_provision_state,
                               self.session, 'manageable')

    def test_timeout(self, mock_fetch):
        self.assertRaises(exceptions.ResourceTimeout,
                          self.node.wait_for_provision_state,
                          self.session, 'manageable', timeout=0.001)

    def test_not_abort_on_failed_state(self, mock_fetch):
        def _get_side_effect(_self, session):
            self.node.provision_state = 'deploy failed'
            self.assertIs(session, self.session)

        mock_fetch.side_effect = _get_side_effect

        self.assertRaises(exceptions.ResourceTimeout,
                          self.node.wait_for_provision_state,
                          self.session, 'manageable', timeout=0.001,
                          abort_on_failed_state=False)


def _fake_assert(self, session, action, expected, error_message=None):
    return expected


@mock.patch.object(node.Node, '_assert_microversion_for', _fake_assert)
@mock.patch.object(node.Node, 'fetch', lambda self, session: self)
@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
class TestNodeSetProvisionState(base.TestCase):

    def setUp(self):
        super(TestNodeSetProvisionState, self).setUp()
        self.node = node.Node(**FAKE)
        self.session = mock.Mock(spec=adapter.Adapter,
                                 default_microversion=None)

    def test_no_arguments(self):
        result = self.node.set_provision_state(self.session, 'active')
        self.assertIs(result, self.node)
        self.session.put.assert_called_once_with(
            'nodes/%s/states/provision' % self.node.id,
            json={'target': 'active'},
            headers=mock.ANY, microversion=None,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_manage(self):
        result = self.node.set_provision_state(self.session, 'manage')
        self.assertIs(result, self.node)
        self.session.put.assert_called_once_with(
            'nodes/%s/states/provision' % self.node.id,
            json={'target': 'manage'},
            headers=mock.ANY, microversion='1.4',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_deploy_with_configdrive(self):
        result = self.node.set_provision_state(self.session, 'active',
                                               config_drive='abcd')
        self.assertIs(result, self.node)
        self.session.put.assert_called_once_with(
            'nodes/%s/states/provision' % self.node.id,
            json={'target': 'active', 'configdrive': 'abcd'},
            headers=mock.ANY, microversion=None,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_rebuild_with_configdrive(self):
        result = self.node.set_provision_state(self.session, 'rebuild',
                                               config_drive='abcd')
        self.assertIs(result, self.node)
        self.session.put.assert_called_once_with(
            'nodes/%s/states/provision' % self.node.id,
            json={'target': 'rebuild', 'configdrive': 'abcd'},
            headers=mock.ANY, microversion='1.35',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_configdrive_as_dict(self):
        for target in ('rebuild', 'active'):
            self.session.put.reset_mock()
            result = self.node.set_provision_state(
                self.session, target, config_drive={'user_data': 'abcd'})
            self.assertIs(result, self.node)
            self.session.put.assert_called_once_with(
                'nodes/%s/states/provision' % self.node.id,
                json={'target': target, 'configdrive': {'user_data': 'abcd'}},
                headers=mock.ANY, microversion='1.56',
                retriable_status_codes=_common.RETRIABLE_STATUS_CODES)


@mock.patch.object(node.Node, '_translate_response', mock.Mock())
@mock.patch.object(node.Node, '_get_session', lambda self, x: x)
@mock.patch.object(node.Node, 'set_provision_state', autospec=True)
class TestNodeCreate(base.TestCase):

    def setUp(self):
        super(TestNodeCreate, self).setUp()
        self.new_state = None
        self.session = mock.Mock(spec=adapter.Adapter)
        self.session.default_microversion = '1.1'
        self.node = node.Node(driver=FAKE['driver'])

        def _change_state(*args, **kwargs):
            self.node.provision_state = self.new_state

        self.session.post.side_effect = _change_state

    def test_available_old_version(self, mock_prov):
        result = self.node.create(self.session)
        self.assertIs(result, self.node)
        self.session.post.assert_called_once_with(
            mock.ANY, json={'driver': FAKE['driver']},
            headers=mock.ANY, microversion=self.session.default_microversion)
        self.assertFalse(mock_prov.called)

    def test_available_new_version(self, mock_prov):
        def _change_state(*args, **kwargs):
            self.node.provision_state = 'manageable'

        self.session.default_microversion = '1.11'
        self.node.provision_state = 'available'
        self.new_state = 'enroll'
        mock_prov.side_effect = _change_state

        result = self.node.create(self.session)
        self.assertIs(result, self.node)
        self.session.post.assert_called_once_with(
            mock.ANY, json={'driver': FAKE['driver']},
            headers=mock.ANY, microversion=self.session.default_microversion)
        mock_prov.assert_has_calls([
            mock.call(self.node, self.session, 'manage', wait=True),
            mock.call(self.node, self.session, 'provide', wait=True)
        ])

    def test_no_enroll_in_old_version(self, mock_prov):
        self.node.provision_state = 'enroll'
        self.assertRaises(exceptions.NotSupported,
                          self.node.create, self.session)
        self.assertFalse(self.session.post.called)
        self.assertFalse(mock_prov.called)

    def test_enroll_new_version(self, mock_prov):
        self.session.default_microversion = '1.11'
        self.node.provision_state = 'enroll'
        self.new_state = 'enroll'

        result = self.node.create(self.session)
        self.assertIs(result, self.node)
        self.session.post.assert_called_once_with(
            mock.ANY, json={'driver': FAKE['driver']},
            headers=mock.ANY, microversion=self.session.default_microversion)
        self.assertFalse(mock_prov.called)

    def test_no_manageable_in_old_version(self, mock_prov):
        self.node.provision_state = 'manageable'
        self.assertRaises(exceptions.NotSupported,
                          self.node.create, self.session)
        self.assertFalse(self.session.post.called)
        self.assertFalse(mock_prov.called)

    def test_manageable_old_version(self, mock_prov):
        self.session.default_microversion = '1.4'
        self.node.provision_state = 'manageable'
        self.new_state = 'available'

        result = self.node.create(self.session)
        self.assertIs(result, self.node)
        self.session.post.assert_called_once_with(
            mock.ANY, json={'driver': FAKE['driver']},
            headers=mock.ANY, microversion=self.session.default_microversion)
        mock_prov.assert_called_once_with(self.node, self.session, 'manage',
                                          wait=True)

    def test_manageable_new_version(self, mock_prov):
        self.session.default_microversion = '1.11'
        self.node.provision_state = 'manageable'
        self.new_state = 'enroll'

        result = self.node.create(self.session)
        self.assertIs(result, self.node)
        self.session.post.assert_called_once_with(
            mock.ANY, json={'driver': FAKE['driver']},
            headers=mock.ANY, microversion=self.session.default_microversion)
        mock_prov.assert_called_once_with(self.node, self.session, 'manage',
                                          wait=True)


@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
@mock.patch.object(node.Node, '_get_session', lambda self, x: x)
class TestNodeVif(base.TestCase):

    def setUp(self):
        super(TestNodeVif, self).setUp()
        self.session = mock.Mock(spec=adapter.Adapter)
        self.session.default_microversion = '1.28'
        self.session.log = mock.Mock()
        self.node = node.Node(id='c29db401-b6a7-4530-af8e-20a720dee946',
                              driver=FAKE['driver'])
        self.vif_id = '714bdf6d-2386-4b5e-bd0d-bc036f04b1ef'

    def test_attach_vif(self):
        self.assertIsNone(self.node.attach_vif(self.session, self.vif_id))
        self.session.post.assert_called_once_with(
            'nodes/%s/vifs' % self.node.id, json={'id': self.vif_id},
            headers=mock.ANY, microversion='1.28',
            retriable_status_codes=[409, 503])

    def test_attach_vif_no_retries(self):
        self.assertIsNone(self.node.attach_vif(self.session, self.vif_id,
                                               retry_on_conflict=False))
        self.session.post.assert_called_once_with(
            'nodes/%s/vifs' % self.node.id, json={'id': self.vif_id},
            headers=mock.ANY, microversion='1.28',
            retriable_status_codes={503})

    def test_detach_vif_existing(self):
        self.assertTrue(self.node.detach_vif(self.session, self.vif_id))
        self.session.delete.assert_called_once_with(
            'nodes/%s/vifs/%s' % (self.node.id, self.vif_id),
            headers=mock.ANY, microversion='1.28',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_detach_vif_missing(self):
        self.session.delete.return_value.status_code = 400
        self.assertFalse(self.node.detach_vif(self.session, self.vif_id))
        self.session.delete.assert_called_once_with(
            'nodes/%s/vifs/%s' % (self.node.id, self.vif_id),
            headers=mock.ANY, microversion='1.28',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_list_vifs(self):
        self.session.get.return_value.json.return_value = {
            'vifs': [
                {'id': '1234'},
                {'id': '5678'},
            ]
        }
        res = self.node.list_vifs(self.session)
        self.assertEqual(['1234', '5678'], res)
        self.session.get.assert_called_once_with(
            'nodes/%s/vifs' % self.node.id,
            headers=mock.ANY, microversion='1.28')

    def test_incompatible_microversion(self):
        self.session.default_microversion = '1.1'
        self.assertRaises(exceptions.NotSupported,
                          self.node.attach_vif,
                          self.session, self.vif_id)
        self.assertRaises(exceptions.NotSupported,
                          self.node.detach_vif,
                          self.session, self.vif_id)
        self.assertRaises(exceptions.NotSupported,
                          self.node.list_vifs,
                          self.session)


@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
@mock.patch.object(node.Node, '_get_session', lambda self, x: x)
class TestNodeValidate(base.TestCase):

    def setUp(self):
        super(TestNodeValidate, self).setUp()
        self.session = mock.Mock(spec=adapter.Adapter)
        self.session.default_microversion = '1.28'
        self.node = node.Node(**FAKE)

    def test_validate_ok(self):
        self.session.get.return_value.json.return_value = {
            'boot': {'result': True},
            'console': {'result': False, 'reason': 'Not configured'},
            'deploy': {'result': True},
            'inspect': {'result': None, 'reason': 'Not supported'},
            'power': {'result': True}
        }
        result = self.node.validate(self.session)
        for iface in ('boot', 'deploy', 'power'):
            self.assertTrue(result[iface].result)
            self.assertFalse(result[iface].reason)
        for iface in ('console', 'inspect'):
            self.assertIsNot(True, result[iface].result)
            self.assertTrue(result[iface].reason)

    def test_validate_failed(self):
        self.session.get.return_value.json.return_value = {
            'boot': {'result': False},
            'console': {'result': False, 'reason': 'Not configured'},
            'deploy': {'result': False, 'reason': 'No deploy for you'},
            'inspect': {'result': None, 'reason': 'Not supported'},
            'power': {'result': True}
        }
        self.assertRaisesRegex(exceptions.ValidationException,
                               'No deploy for you',
                               self.node.validate, self.session)

    def test_validate_no_failure(self):
        self.session.get.return_value.json.return_value = {
            'boot': {'result': False},
            'console': {'result': False, 'reason': 'Not configured'},
            'deploy': {'result': False, 'reason': 'No deploy for you'},
            'inspect': {'result': None, 'reason': 'Not supported'},
            'power': {'result': True}
        }
        result = self.node.validate(self.session, required=None)
        self.assertTrue(result['power'].result)
        self.assertFalse(result['power'].reason)
        for iface in ('deploy', 'console', 'inspect'):
            self.assertIsNot(True, result[iface].result)
            self.assertTrue(result[iface].reason)
        # Reason can be empty
        self.assertFalse(result['boot'].result)
        self.assertIsNone(result['boot'].reason)


@mock.patch('time.sleep', lambda _t: None)
@mock.patch.object(node.Node, 'fetch', autospec=True)
class TestNodeWaitForReservation(base.TestCase):

    def setUp(self):
        super(TestNodeWaitForReservation, self).setUp()
        self.session = mock.Mock(spec=adapter.Adapter)
        self.session.default_microversion = '1.6'
        self.session.log = mock.Mock()
        self.node = node.Node(**FAKE)

    def test_no_reservation(self, mock_fetch):
        self.node.reservation = None
        node = self.node.wait_for_reservation(None)
        self.assertIs(node, self.node)
        self.assertFalse(mock_fetch.called)

    def test_reservation(self, mock_fetch):
        self.node.reservation = 'example.com'

        def _side_effect(node, session):
            if self.node.reservation == 'example.com':
                self.node.reservation = 'example2.com'
            else:
                self.node.reservation = None

        mock_fetch.side_effect = _side_effect
        node = self.node.wait_for_reservation(self.session)
        self.assertIs(node, self.node)
        self.assertEqual(2, mock_fetch.call_count)

    def test_timeout(self, mock_fetch):
        self.node.reservation = 'example.com'

        self.assertRaises(exceptions.ResourceTimeout,
                          self.node.wait_for_reservation,
                          self.session, timeout=0.001)
        mock_fetch.assert_called_with(self.node, self.session)


@mock.patch.object(node.Node, '_assert_microversion_for', _fake_assert)
@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
class TestNodeSetPowerState(base.TestCase):

    def setUp(self):
        super(TestNodeSetPowerState, self).setUp()
        self.node = node.Node(**FAKE)
        self.session = mock.Mock(spec=adapter.Adapter,
                                 default_microversion=None)

    def test_power_on(self):
        self.node.set_power_state(self.session, 'power on')
        self.session.put.assert_called_once_with(
            'nodes/%s/states/power' % FAKE['uuid'],
            json={'target': 'power on'},
            headers=mock.ANY,
            microversion=None,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_soft_power_on(self):
        self.node.set_power_state(self.session, 'soft power off')
        self.session.put.assert_called_once_with(
            'nodes/%s/states/power' % FAKE['uuid'],
            json={'target': 'soft power off'},
            headers=mock.ANY,
            microversion='1.27',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)


@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
@mock.patch.object(node.Node, '_translate_response', mock.Mock())
@mock.patch.object(node.Node, '_get_session', lambda self, x: x)
class TestNodeMaintenance(base.TestCase):

    def setUp(self):
        super(TestNodeMaintenance, self).setUp()
        self.node = node.Node.existing(**FAKE)
        self.session = mock.Mock(spec=adapter.Adapter,
                                 default_microversion='1.1',
                                 retriable_status_codes=None)

    def test_set(self):
        self.node.set_maintenance(self.session)
        self.session.put.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json={'reason': None},
            headers=mock.ANY,
            microversion=mock.ANY)

    def test_set_with_reason(self):
        self.node.set_maintenance(self.session, 'No work on Monday')
        self.session.put.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json={'reason': 'No work on Monday'},
            headers=mock.ANY,
            microversion=mock.ANY)

    def test_unset(self):
        self.node.unset_maintenance(self.session)
        self.session.delete.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json=None,
            headers=mock.ANY,
            microversion=mock.ANY)

    def test_set_via_update(self):
        self.node.is_maintenance = True
        self.node.commit(self.session)
        self.session.put.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json={'reason': None},
            headers=mock.ANY,
            microversion=mock.ANY)

        self.assertFalse(self.session.patch.called)

    def test_set_with_reason_via_update(self):
        self.node.is_maintenance = True
        self.node.maintenance_reason = 'No work on Monday'
        self.node.commit(self.session)
        self.session.put.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json={'reason': 'No work on Monday'},
            headers=mock.ANY,
            microversion=mock.ANY)
        self.assertFalse(self.session.patch.called)

    def test_set_with_other_fields(self):
        self.node.is_maintenance = True
        self.node.name = 'lazy-3000'
        self.node.commit(self.session)
        self.session.put.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json={'reason': None},
            headers=mock.ANY,
            microversion=mock.ANY)

        self.session.patch.assert_called_once_with(
            'nodes/%s' % self.node.id,
            json=[{'path': '/name', 'op': 'replace', 'value': 'lazy-3000'}],
            headers=mock.ANY,
            microversion=mock.ANY)

    def test_set_with_reason_and_other_fields(self):
        self.node.is_maintenance = True
        self.node.maintenance_reason = 'No work on Monday'
        self.node.name = 'lazy-3000'
        self.node.commit(self.session)
        self.session.put.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json={'reason': 'No work on Monday'},
            headers=mock.ANY,
            microversion=mock.ANY)

        self.session.patch.assert_called_once_with(
            'nodes/%s' % self.node.id,
            json=[{'path': '/name', 'op': 'replace', 'value': 'lazy-3000'}],
            headers=mock.ANY,
            microversion=mock.ANY)

    def test_no_reason_without_maintenance(self):
        self.node.maintenance_reason = 'Can I?'
        self.assertRaises(ValueError, self.node.commit, self.session)
        self.assertFalse(self.session.put.called)
        self.assertFalse(self.session.patch.called)

    def test_set_unset_maintenance(self):
        self.node.is_maintenance = True
        self.node.maintenance_reason = 'No work on Monday'
        self.node.commit(self.session)

        self.session.put.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json={'reason': 'No work on Monday'},
            headers=mock.ANY,
            microversion=mock.ANY)

        self.node.is_maintenance = False
        self.node.commit(self.session)
        self.assertIsNone(self.node.maintenance_reason)

        self.session.delete.assert_called_once_with(
            'nodes/%s/maintenance' % self.node.id,
            json=None,
            headers=mock.ANY,
            microversion=mock.ANY)


@mock.patch.object(node.Node, 'fetch', lambda self, session: self)
@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
class TestNodeSetBootDevice(base.TestCase):

    def setUp(self):
        super(TestNodeSetBootDevice, self).setUp()
        self.node = node.Node(**FAKE)
        self.session = mock.Mock(spec=adapter.Adapter,
                                 default_microversion='1.1')

    def test_node_set_boot_device(self):
        self.node.set_boot_device(self.session, 'pxe', persistent=False)
        self.session.put.assert_called_once_with(
            'nodes/%s/management/boot_device' % self.node.id,
            json={'boot_device': 'pxe', 'persistent': False},
            headers=mock.ANY, microversion=mock.ANY,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)


@mock.patch.object(node.Node, 'fetch', lambda self, session: self)
@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
class TestNodeTraits(base.TestCase):

    def setUp(self):
        super(TestNodeTraits, self).setUp()
        self.node = node.Node(**FAKE)
        self.session = mock.Mock(spec=adapter.Adapter,
                                 default_microversion='1.37')
        self.session.log = mock.Mock()

    def test_node_add_trait(self):
        self.node.add_trait(self.session, 'CUSTOM_FAKE')
        self.session.put.assert_called_once_with(
            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_FAKE'),
            json=None,
            headers=mock.ANY, microversion='1.37',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_remove_trait(self):
        self.node.remove_trait(self.session, 'CUSTOM_FAKE')
        self.session.delete.assert_called_once_with(
            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_FAKE'),
            headers=mock.ANY, microversion='1.37',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_remove_trait_missing(self):
        self.session.delete.return_value.status_code = 400
        self.assertFalse(self.node.remove_trait(self.session,
                                                'CUSTOM_MISSING'))
        self.session.delete.assert_called_once_with(
            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_MISSING'),
            headers=mock.ANY, microversion='1.37',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

    def test_set_traits(self):
        traits = ['CUSTOM_FAKE', 'CUSTOM_REAL', 'CUSTOM_MISSING']
        self.node.set_traits(self.session, traits)
        self.session.put.assert_called_once_with(
            'nodes/%s/traits' % self.node.id,
            json={'traits': ['CUSTOM_FAKE', 'CUSTOM_REAL', 'CUSTOM_MISSING']},
            headers=mock.ANY, microversion='1.37',
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import floating_ip
from openstack.network.v2 import network
from openstack.network.v2 import port_forwarding as _port_forwarding
from openstack.network.v2 import port
from openstack.network.v2 import router
from openstack.network.v2 import subnet
from openstack.tests.functional import base


class TestPortForwarding(base.BaseFunctionalTest):

    IPV4 = 4
    FIP_ID = None
    EXT_CIDR = "10.100.0.0/24"
    INT_CIDR = "10.101.0.0/24"
    EXT_NET_ID = None
    INT_NET_ID = None
    EXT_SUB_ID = None
    INT_SUB_ID = None
    ROT_ID = None

    INTERNAL_PORT_ID = None
    INTERNAL_IP_ADDRESS = None
    INTERNAL_PORT = 8080
    EXTERNAL_PORT = 80
    PROTOCOL = "tcp"

    def setUp(self):
        super(TestPortForwarding, self).setUp()

        if not self.conn.network.find_extension('floating-ip-port-forwarding'):
            self.skipTest('Floating IP Port Forwarding extension disabled')

        self.ROT_NAME = self.getUniqueString()
        self.EXT_NET_NAME = self.getUniqueString()
        self.EXT_SUB_NAME = self.getUniqueString()
        self.INT_NET_NAME = self.getUniqueString()
        self.INT_SUB_NAME = self.getUniqueString()
        # Create Exeternal Network
        args = {'router:external': True}
        net = self._create_network(self.EXT_NET_NAME, **args)
        self.EXT_NET_ID = net.id
        sub = self._create_subnet(
            self.EXT_SUB_NAME, self.EXT_NET_ID, self.EXT_CIDR)
        self.EXT_SUB_ID = sub.id
        # Create Internal Network
        net = self._create_network(self.INT_NET_NAME)
        self.INT_NET_ID = net.id
        sub = self._create_subnet(
            self.INT_SUB_NAME, self.INT_NET_ID, self.INT_CIDR)
        self.INT_SUB_ID = sub.id
        # Create Router
        args = {'external_gateway_info': {'network_id': self.EXT_NET_ID}}
        sot = self.conn.network.create_router(name=self.ROT_NAME, **args)
        assert isinstance(sot, router.Router)
        self.assertEqual(self.ROT_NAME, sot.name)
        self.ROT_ID = sot.id
        self.ROT = sot
        # Add Router's Interface to Internal Network
        sot = self.ROT.add_interface(
            self.conn.network, subnet_id=self.INT_SUB_ID)
        self.assertEqual(sot['subnet_id'], self.INT_SUB_ID)
        # Create Port in Internal Network
        prt = self.conn.network.create_port(network_id=self.INT_NET_ID)
        assert isinstance(prt, port.Port)
        self.INTERNAL_PORT_ID = prt.id
        self.INTERNAL_IP_ADDRESS = prt.fixed_ips[0]['ip_address']
        # Create Floating IP.
        fip = self.conn.network.create_ip(
            floating_network_id=self.EXT_NET_ID)
        assert isinstance(fip, floating_ip.FloatingIP)
        self.FIP_ID = fip.id
        # Create Port Forwarding
        pf = self.conn.network.create_port_forwarding(
            floatingip_id=self.FIP_ID,
            internal_port_id=self.INTERNAL_PORT_ID,
            internal_ip_address=self.INTERNAL_IP_ADDRESS,
            internal_port=self.INTERNAL_PORT,
            external_port=self.EXTERNAL_PORT,
            protocol=self.PROTOCOL)
        assert isinstance(pf, _port_forwarding.PortForwarding)
        self.PF = pf

    def tearDown(self):
        sot = self.conn.network.delete_port_forwarding(
            self.PF, self.FIP_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_ip(self.FIP_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_port(
            self.INTERNAL_PORT_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.ROT.remove_interface(
            self.conn.network, subnet_id=self.INT_SUB_ID)
        self.assertEqual(sot['subnet_id'], self.INT_SUB_ID)
        sot = self.conn.network.delete_router(
            self.ROT_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_subnet(
            self.EXT_SUB_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_network(
            self.EXT_NET_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_subnet(
            self.INT_SUB_ID, ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_network(
            self.INT_NET_ID, ignore_missing=False)
        self.assertIsNone(sot)
        super(TestPortForwarding, self).tearDown()

    def _create_network(self, name, **args):
        self.name = name
        net = self.conn.network.create_network(name=name, **args)
        assert isinstance(net, network.Network)
        self.assertEqual(self.name, net.name)
        return net

    def _create_subnet(self, name, net_id, cidr):
        self.name = name
        self.net_id = net_id
        self.cidr = cidr
        sub = self.conn.network.create_subnet(
            name=self.name,
            ip_version=self.IPV4,
            network_id=self.net_id,
            cidr=self.cidr)
        assert isinstance(sub, subnet.Subnet)
        self.assertEqual(self.name, sub.name)
        return sub

    def test_find(self):
        sot = self.conn.network.find_port_forwarding(
            self.PF.id, self.FIP_ID)
        self.assertEqual(self.INTERNAL_PORT_ID, sot.internal_port_id)
        self.assertEqual(self.INTERNAL_IP_ADDRESS, sot.internal_ip_address)
        self.assertEqual(self.INTERNAL_PORT, sot.internal_port)
        self.assertEqual(self.EXTERNAL_PORT, sot.external_port)
        self.assertEqual(self.PROTOCOL, sot.protocol)

    def test_get(self):
        sot = self.conn.network.get_port_forwarding(
            self.PF, self.FIP_ID)
        self.assertEqual(self.INTERNAL_PORT_ID, sot.internal_port_id)
        self.assertEqual(self.INTERNAL_IP_ADDRESS, sot.internal_ip_address)
        self.assertEqual(self.INTERNAL_PORT, sot.internal_port)
        self.assertEqual(self.EXTERNAL_PORT, sot.external_port)
        self.assertEqual(self.PROTOCOL, sot.protocol)

    def test_list(self):
        pf_ids = [o.id for o in
                  self.conn.network.port_forwardings(self.FIP_ID)]
        self.assertIn(self.PF.id, pf_ids)

    def test_update(self):
        NEW_EXTERNAL_PORT = 90
        sot = self.conn.network.update_port_forwarding(
            self.PF.id,
            self.FIP_ID,
            external_port=NEW_EXTERNAL_PORT)
        self.assertEqual(NEW_EXTERNAL_PORT, sot.external_port)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.load_balancer.v2 import flavor
from openstack.load_balancer.v2 import flavor_profile
from openstack.load_balancer.v2 import health_monitor
from openstack.load_balancer.v2 import l7_policy
from openstack.load_balancer.v2 import l7_rule
from openstack.load_balancer.v2 import listener
from openstack.load_balancer.v2 import load_balancer
from openstack.load_balancer.v2 import member
from openstack.load_balancer.v2 import pool
from openstack.load_balancer.v2 import quota
from openstack.tests.functional import base


class TestLoadBalancer(base.BaseFunctionalTest):

    HM_ID = None
    L7POLICY_ID = None
    LB_ID = None
    LISTENER_ID = None
    MEMBER_ID = None
    POOL_ID = None
    VIP_SUBNET_ID = None
    PROJECT_ID = None
    FLAVOR_PROFILE_ID = None
    FLAVOR_ID = None
    AMPHORA_ID = None
    PROTOCOL = 'HTTP'
    PROTOCOL_PORT = 80
    LB_ALGORITHM = 'ROUND_ROBIN'
    MEMBER_ADDRESS = '192.0.2.16'
    WEIGHT = 10
    DELAY = 2
    TIMEOUT = 1
    MAX_RETRY = 3
    HM_TYPE = 'HTTP'
    ACTION = 'REDIRECT_TO_URL'
    REDIRECT_URL = 'http://www.example.com'
    COMPARE_TYPE = 'CONTAINS'
    L7RULE_TYPE = 'HOST_NAME'
    L7RULE_VALUE = 'example'
    AMPHORA = 'amphora'
    FLAVOR_DATA = '{"loadbalancer_topology": "SINGLE"}'
    DESCRIPTION = 'Test description'

    _wait_for_timeout_key = 'OPENSTACKSDK_FUNC_TEST_TIMEOUT_LOAD_BALANCER'

    # TODO(shade): Creating load balancers can be slow on some hosts due to
    #              nova instance boot times (up to ten minutes). This used to
    #              use setUpClass, but that's a whole other pile of bad, so
    #              we may need to engineer something pleasing here.
    def setUp(self):
        super(TestLoadBalancer, self).setUp()
        self.require_service('load-balancer')

        self.HM_NAME = self.getUniqueString()
        self.L7POLICY_NAME = self.getUniqueString()
        self.LB_NAME = self.getUniqueString()
        self.LISTENER_NAME = self.getUniqueString()
        self.MEMBER_NAME = self.getUniqueString()
        self.POOL_NAME = self.getUniqueString()
        self.UPDATE_NAME = self.getUniqueString()
        self.FLAVOR_PROFILE_NAME = self.getUniqueString()
        self.FLAVOR_NAME = self.getUniqueString()
        subnets = list(self.conn.network.subnets())
        self.VIP_SUBNET_ID = subnets[0].id
        self.PROJECT_ID = self.conn.session.get_project_id()
        test_quota = self.conn.load_balancer.update_quota(
            self.PROJECT_ID, **{'load_balancer': 100,
                                'pool': 100,
                                'listener': 100,
                                'health_monitor': 100,
                                'member': 100})
        assert isinstance(test_quota, quota.Quota)
        self.assertEqual(self.PROJECT_ID, test_quota.id)

        test_profile = self.conn.load_balancer.create_flavor_profile(
            name=self.FLAVOR_PROFILE_NAME, provider_name=self.AMPHORA,
            flavor_data=self.FLAVOR_DATA)
        assert isinstance(test_profile, flavor_profile.FlavorProfile)
        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_profile.name)
        self.FLAVOR_PROFILE_ID = test_profile.id

        test_flavor = self.conn.load_balancer.create_flavor(
            name=self.FLAVOR_NAME, flavor_profile_id=self.FLAVOR_PROFILE_ID,
            is_enabled=True, description=self.DESCRIPTION)
        assert isinstance(test_flavor, flavor.Flavor)
        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)
        self.FLAVOR_ID = test_flavor.id

        test_lb = self.conn.load_balancer.create_load_balancer(
            name=self.LB_NAME, vip_subnet_id=self.VIP_SUBNET_ID,
            project_id=self.PROJECT_ID)
        assert isinstance(test_lb, load_balancer.LoadBalancer)
        self.assertEqual(self.LB_NAME, test_lb.name)
        # Wait for the LB to go ACTIVE.  On non-virtualization enabled hosts
        # it can take nova up to ten minutes to boot a VM.
        self.conn.load_balancer.wait_for_load_balancer(
            test_lb.id, interval=1,
            wait=self._wait_for_timeout)
        self.LB_ID = test_lb.id

        amphorae = self.conn.load_balancer.amphorae(loadbalancer_id=self.LB_ID)
        for amp in amphorae:
            self.AMPHORA_ID = amp.id

        test_listener = self.conn.load_balancer.create_listener(
            name=self.LISTENER_NAME, protocol=self.PROTOCOL,
            protocol_port=self.PROTOCOL_PORT, loadbalancer_id=self.LB_ID)
        assert isinstance(test_listener, listener.Listener)
        self.assertEqual(self.LISTENER_NAME, test_listener.name)
        self.LISTENER_ID = test_listener.id
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        test_pool = self.conn.load_balancer.create_pool(
            name=self.POOL_NAME, protocol=self.PROTOCOL,
            lb_algorithm=self.LB_ALGORITHM, listener_id=self.LISTENER_ID)
        assert isinstance(test_pool, pool.Pool)
        self.assertEqual(self.POOL_NAME, test_pool.name)
        self.POOL_ID = test_pool.id
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        test_member = self.conn.load_balancer.create_member(
            pool=self.POOL_ID, name=self.MEMBER_NAME,
            address=self.MEMBER_ADDRESS,
            protocol_port=self.PROTOCOL_PORT, weight=self.WEIGHT)
        assert isinstance(test_member, member.Member)
        self.assertEqual(self.MEMBER_NAME, test_member.name)
        self.MEMBER_ID = test_member.id
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        test_hm = self.conn.load_balancer.create_health_monitor(
            pool_id=self.POOL_ID, name=self.HM_NAME, delay=self.DELAY,
            timeout=self.TIMEOUT, max_retries=self.MAX_RETRY,
            type=self.HM_TYPE)
        assert isinstance(test_hm, health_monitor.HealthMonitor)
        self.assertEqual(self.HM_NAME, test_hm.name)
        self.HM_ID = test_hm.id
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        test_l7policy = self.conn.load_balancer.create_l7_policy(
            listener_id=self.LISTENER_ID, name=self.L7POLICY_NAME,
            action=self.ACTION, redirect_url=self.REDIRECT_URL)
        assert isinstance(test_l7policy, l7_policy.L7Policy)
        self.assertEqual(self.L7POLICY_NAME, test_l7policy.name)
        self.L7POLICY_ID = test_l7policy.id
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        test_l7rule = self.conn.load_balancer.create_l7_rule(
            l7_policy=self.L7POLICY_ID, compare_type=self.COMPARE_TYPE,
            type=self.L7RULE_TYPE, value=self.L7RULE_VALUE)
        assert isinstance(test_l7rule, l7_rule.L7Rule)
        self.assertEqual(self.COMPARE_TYPE, test_l7rule.compare_type)
        self.L7RULE_ID = test_l7rule.id
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

    def tearDown(self):
        self.conn.load_balancer.get_load_balancer(self.LB_ID)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        self.conn.load_balancer.delete_quota(self.PROJECT_ID,
                                             ignore_missing=False)

        self.conn.load_balancer.delete_l7_rule(
            self.L7RULE_ID, l7_policy=self.L7POLICY_ID, ignore_missing=False)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        self.conn.load_balancer.delete_l7_policy(
            self.L7POLICY_ID, ignore_missing=False)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        self.conn.load_balancer.delete_health_monitor(
            self.HM_ID, ignore_missing=False)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        self.conn.load_balancer.delete_member(
            self.MEMBER_ID, self.POOL_ID, ignore_missing=False)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        self.conn.load_balancer.delete_pool(self.POOL_ID, ignore_missing=False)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        self.conn.load_balancer.delete_listener(self.LISTENER_ID,
                                                ignore_missing=False)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)

        self.conn.load_balancer.delete_load_balancer(
            self.LB_ID, ignore_missing=False)
        super(TestLoadBalancer, self).tearDown()

        self.conn.load_balancer.delete_flavor(self.FLAVOR_ID,
                                              ignore_missing=False)

        self.conn.load_balancer.delete_flavor_profile(self.FLAVOR_PROFILE_ID,
                                                      ignore_missing=False)

    def test_lb_find(self):
        test_lb = self.conn.load_balancer.find_load_balancer(self.LB_NAME)
        self.assertEqual(self.LB_ID, test_lb.id)

    def test_lb_get(self):
        test_lb = self.conn.load_balancer.get_load_balancer(self.LB_ID)
        self.assertEqual(self.LB_NAME, test_lb.name)
        self.assertEqual(self.LB_ID, test_lb.id)
        self.assertEqual(self.VIP_SUBNET_ID, test_lb.vip_subnet_id)

    def test_lb_get_stats(self):
        test_lb_stats = self.conn.load_balancer.get_load_balancer_statistics(
            self.LB_ID)
        self.assertEqual(0, test_lb_stats.active_connections)
        self.assertEqual(0, test_lb_stats.bytes_in)
        self.assertEqual(0, test_lb_stats.bytes_out)
        self.assertEqual(0, test_lb_stats.request_errors)
        self.assertEqual(0, test_lb_stats.total_connections)

    def test_lb_list(self):
        names = [lb.name for lb in self.conn.load_balancer.load_balancers()]
        self.assertIn(self.LB_NAME, names)

    def test_lb_update(self):
        self.conn.load_balancer.update_load_balancer(
            self.LB_ID, name=self.UPDATE_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_lb = self.conn.load_balancer.get_load_balancer(self.LB_ID)
        self.assertEqual(self.UPDATE_NAME, test_lb.name)

        self.conn.load_balancer.update_load_balancer(
            self.LB_ID, name=self.LB_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_lb = self.conn.load_balancer.get_load_balancer(self.LB_ID)
        self.assertEqual(self.LB_NAME, test_lb.name)

    def test_lb_failover(self):
        self.conn.load_balancer.failover_load_balancer(self.LB_ID)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_lb = self.conn.load_balancer.get_load_balancer(self.LB_ID)
        self.assertEqual(self.LB_NAME, test_lb.name)

    def test_listener_find(self):
        test_listener = self.conn.load_balancer.find_listener(
            self.LISTENER_NAME)
        self.assertEqual(self.LISTENER_ID, test_listener.id)

    def test_listener_get(self):
        test_listener = self.conn.load_balancer.get_listener(self.LISTENER_ID)
        self.assertEqual(self.LISTENER_NAME, test_listener.name)
        self.assertEqual(self.LISTENER_ID, test_listener.id)
        self.assertEqual(self.PROTOCOL, test_listener.protocol)
        self.assertEqual(self.PROTOCOL_PORT, test_listener.protocol_port)

    def test_listener_get_stats(self):
        test_listener_stats = self.conn.load_balancer.get_listener_statistics(
            self.LISTENER_ID)
        self.assertEqual(0, test_listener_stats.active_connections)
        self.assertEqual(0, test_listener_stats.bytes_in)
        self.assertEqual(0, test_listener_stats.bytes_out)
        self.assertEqual(0, test_listener_stats.request_errors)
        self.assertEqual(0, test_listener_stats.total_connections)

    def test_listener_list(self):
        names = [ls.name for ls in self.conn.load_balancer.listeners()]
        self.assertIn(self.LISTENER_NAME, names)

    def test_listener_update(self):
        self.conn.load_balancer.get_load_balancer(self.LB_ID)

        self.conn.load_balancer.update_listener(
            self.LISTENER_ID, name=self.UPDATE_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_listener = self.conn.load_balancer.get_listener(self.LISTENER_ID)
        self.assertEqual(self.UPDATE_NAME, test_listener.name)

        self.conn.load_balancer.update_listener(
            self.LISTENER_ID, name=self.LISTENER_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_listener = self.conn.load_balancer.get_listener(self.LISTENER_ID)
        self.assertEqual(self.LISTENER_NAME, test_listener.name)

    def test_pool_find(self):
        test_pool = self.conn.load_balancer.find_pool(self.POOL_NAME)
        self.assertEqual(self.POOL_ID, test_pool.id)

    def test_pool_get(self):
        test_pool = self.conn.load_balancer.get_pool(self.POOL_ID)
        self.assertEqual(self.POOL_NAME, test_pool.name)
        self.assertEqual(self.POOL_ID, test_pool.id)
        self.assertEqual(self.PROTOCOL, test_pool.protocol)

    def test_pool_list(self):
        names = [pool.name for pool in self.conn.load_balancer.pools()]
        self.assertIn(self.POOL_NAME, names)

    def test_pool_update(self):
        self.conn.load_balancer.get_load_balancer(self.LB_ID)

        self.conn.load_balancer.update_pool(self.POOL_ID,
                                            name=self.UPDATE_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_pool = self.conn.load_balancer.get_pool(self.POOL_ID)
        self.assertEqual(self.UPDATE_NAME, test_pool.name)

        self.conn.load_balancer.update_pool(self.POOL_ID,
                                            name=self.POOL_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_pool = self.conn.load_balancer.get_pool(self.POOL_ID)
        self.assertEqual(self.POOL_NAME, test_pool.name)

    def test_member_find(self):
        test_member = self.conn.load_balancer.find_member(self.MEMBER_NAME,
                                                          self.POOL_ID)
        self.assertEqual(self.MEMBER_ID, test_member.id)

    def test_member_get(self):
        test_member = self.conn.load_balancer.get_member(self.MEMBER_ID,
                                                         self.POOL_ID)
        self.assertEqual(self.MEMBER_NAME, test_member.name)
        self.assertEqual(self.MEMBER_ID, test_member.id)
        self.assertEqual(self.MEMBER_ADDRESS, test_member.address)
        self.assertEqual(self.PROTOCOL_PORT, test_member.protocol_port)
        self.assertEqual(self.WEIGHT, test_member.weight)

    def test_member_list(self):
        names = [mb.name for mb in self.conn.load_balancer.members(
            self.POOL_ID)]
        self.assertIn(self.MEMBER_NAME, names)

    def test_member_update(self):
        self.conn.load_balancer.get_load_balancer(self.LB_ID)

        self.conn.load_balancer.update_member(self.MEMBER_ID, self.POOL_ID,
                                              name=self.UPDATE_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_member = self.conn.load_balancer.get_member(self.MEMBER_ID,
                                                         self.POOL_ID)
        self.assertEqual(self.UPDATE_NAME, test_member.name)

        self.conn.load_balancer.update_member(self.MEMBER_ID, self.POOL_ID,
                                              name=self.MEMBER_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_member = self.conn.load_balancer.get_member(self.MEMBER_ID,
                                                         self.POOL_ID)
        self.assertEqual(self.MEMBER_NAME, test_member.name)

    def test_health_monitor_find(self):
        test_hm = self.conn.load_balancer.find_health_monitor(self.HM_NAME)
        self.assertEqual(self.HM_ID, test_hm.id)

    def test_health_monitor_get(self):
        test_hm = self.conn.load_balancer.get_health_monitor(self.HM_ID)
        self.assertEqual(self.HM_NAME, test_hm.name)
        self.assertEqual(self.HM_ID, test_hm.id)
        self.assertEqual(self.DELAY, test_hm.delay)
        self.assertEqual(self.TIMEOUT, test_hm.timeout)
        self.assertEqual(self.MAX_RETRY, test_hm.max_retries)
        self.assertEqual(self.HM_TYPE, test_hm.type)

    def test_health_monitor_list(self):
        names = [hm.name for hm in self.conn.load_balancer.health_monitors()]
        self.assertIn(self.HM_NAME, names)

    def test_health_monitor_update(self):
        self.conn.load_balancer.get_load_balancer(self.LB_ID)

        self.conn.load_balancer.update_health_monitor(self.HM_ID,
                                                      name=self.UPDATE_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_hm = self.conn.load_balancer.get_health_monitor(self.HM_ID)
        self.assertEqual(self.UPDATE_NAME, test_hm.name)

        self.conn.load_balancer.update_health_monitor(self.HM_ID,
                                                      name=self.HM_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_hm = self.conn.load_balancer.get_health_monitor(self.HM_ID)
        self.assertEqual(self.HM_NAME, test_hm.name)

    def test_l7_policy_find(self):
        test_l7_policy = self.conn.load_balancer.find_l7_policy(
            self.L7POLICY_NAME)
        self.assertEqual(self.L7POLICY_ID, test_l7_policy.id)

    def test_l7_policy_get(self):
        test_l7_policy = self.conn.load_balancer.get_l7_policy(
            self.L7POLICY_ID)
        self.assertEqual(self.L7POLICY_NAME, test_l7_policy.name)
        self.assertEqual(self.L7POLICY_ID, test_l7_policy.id)
        self.assertEqual(self.ACTION, test_l7_policy.action)

    def test_l7_policy_list(self):
        names = [l7.name for l7 in self.conn.load_balancer.l7_policies()]
        self.assertIn(self.L7POLICY_NAME, names)

    def test_l7_policy_update(self):
        self.conn.load_balancer.get_load_balancer(self.LB_ID)

        self.conn.load_balancer.update_l7_policy(
            self.L7POLICY_ID, name=self.UPDATE_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_l7_policy = self.conn.load_balancer.get_l7_policy(
            self.L7POLICY_ID)
        self.assertEqual(self.UPDATE_NAME, test_l7_policy.name)

        self.conn.load_balancer.update_l7_policy(self.L7POLICY_ID,
                                                 name=self.L7POLICY_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_l7_policy = self.conn.load_balancer.get_l7_policy(
            self.L7POLICY_ID)
        self.assertEqual(self.L7POLICY_NAME, test_l7_policy.name)

    def test_l7_rule_find(self):
        test_l7_rule = self.conn.load_balancer.find_l7_rule(
            self.L7RULE_ID, self.L7POLICY_ID)
        self.assertEqual(self.L7RULE_ID, test_l7_rule.id)
        self.assertEqual(self.L7RULE_TYPE, test_l7_rule.type)

    def test_l7_rule_get(self):
        test_l7_rule = self.conn.load_balancer.get_l7_rule(
            self.L7RULE_ID, l7_policy=self.L7POLICY_ID)
        self.assertEqual(self.L7RULE_ID, test_l7_rule.id)
        self.assertEqual(self.COMPARE_TYPE, test_l7_rule.compare_type)
        self.assertEqual(self.L7RULE_TYPE, test_l7_rule.type)
        self.assertEqual(self.L7RULE_VALUE, test_l7_rule.rule_value)

    def test_l7_rule_list(self):
        ids = [l7.id for l7 in self.conn.load_balancer.l7_rules(
            l7_policy=self.L7POLICY_ID)]
        self.assertIn(self.L7RULE_ID, ids)

    def test_l7_rule_update(self):
        self.conn.load_balancer.get_load_balancer(self.LB_ID)

        self.conn.load_balancer.update_l7_rule(self.L7RULE_ID,
                                               l7_policy=self.L7POLICY_ID,
                                               rule_value=self.UPDATE_NAME)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_l7_rule = self.conn.load_balancer.get_l7_rule(
            self.L7RULE_ID, l7_policy=self.L7POLICY_ID)
        self.assertEqual(self.UPDATE_NAME, test_l7_rule.rule_value)

        self.conn.load_balancer.update_l7_rule(self.L7RULE_ID,
                                               l7_policy=self.L7POLICY_ID,
                                               rule_value=self.L7RULE_VALUE)
        self.conn.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout)
        test_l7_rule = self.conn.load_balancer.get_l7_rule(
            self.L7RULE_ID, l7_policy=self.L7POLICY_ID,)
        self.assertEqual(self.L7RULE_VALUE, test_l7_rule.rule_value)

    def test_quota_list(self):
        for qot in self.conn.load_balancer.quotas():
            self.assertIsNotNone(qot.project_id)

    def test_quota_get(self):
        test_quota = self.conn.load_balancer.get_quota(self.PROJECT_ID)
        self.assertEqual(self.PROJECT_ID, test_quota.id)

    def test_quota_update(self):
        attrs = {'load_balancer': 12345, 'pool': 67890}
        for project_quota in self.conn.load_balancer.quotas():
            self.conn.load_balancer.update_quota(project_quota, **attrs)
            new_quota = self.conn.load_balancer.get_quota(
                project_quota.project_id)
            self.assertEqual(12345, new_quota.load_balancers)
            self.assertEqual(67890, new_quota.pools)

    def test_default_quota(self):
        self.conn.load_balancer.get_quota_default()

    def test_providers(self):
        providers = self.conn.load_balancer.providers()
        # Make sure our default provider is in the list
        self.assertTrue(
            any(prov['name'] == self.AMPHORA for prov in providers))

    def test_provider_flavor_capabilities(self):
        capabilities = self.conn.load_balancer.provider_flavor_capabilities(
            self.AMPHORA)
        # Make sure a known capability is in the default provider
        self.assertTrue(any(
            cap['name'] == 'loadbalancer_topology' for cap in capabilities))

    def test_flavor_profile_find(self):
        test_profile = self.conn.load_balancer.find_flavor_profile(
            self.FLAVOR_PROFILE_NAME)
        self.assertEqual(self.FLAVOR_PROFILE_ID, test_profile.id)

    def test_flavor_profile_get(self):
        test_flavor_profile = self.conn.load_balancer.get_flavor_profile(
            self.FLAVOR_PROFILE_ID)
        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_flavor_profile.name)
        self.assertEqual(self.FLAVOR_PROFILE_ID, test_flavor_profile.id)
        self.assertEqual(self.AMPHORA, test_flavor_profile.provider_name)
        self.assertEqual(self.FLAVOR_DATA, test_flavor_profile.flavor_data)

    def test_flavor_profile_list(self):
        names = [fv.name for fv in self.conn.load_balancer.flavor_profiles()]
        self.assertIn(self.FLAVOR_PROFILE_NAME, names)

    def test_flavor_profile_update(self):
        self.conn.load_balancer.update_flavor_profile(
            self.FLAVOR_PROFILE_ID, name=self.UPDATE_NAME)
        test_flavor_profile = self.conn.load_balancer.get_flavor_profile(
            self.FLAVOR_PROFILE_ID)
        self.assertEqual(self.UPDATE_NAME, test_flavor_profile.name)

        self.conn.load_balancer.update_flavor_profile(
            self.FLAVOR_PROFILE_ID, name=self.FLAVOR_PROFILE_NAME)
        test_flavor_profile = self.conn.load_balancer.get_flavor_profile(
            self.FLAVOR_PROFILE_ID)
        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_flavor_profile.name)

    def test_flavor_find(self):
        test_flavor = self.conn.load_balancer.find_flavor(self.FLAVOR_NAME)
        self.assertEqual(self.FLAVOR_ID, test_flavor.id)

    def test_flavor_get(self):
        test_flavor = self.conn.load_balancer.get_flavor(self.FLAVOR_ID)
        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)
        self.assertEqual(self.FLAVOR_ID, test_flavor.id)
        self.assertEqual(self.DESCRIPTION, test_flavor.description)
        self.assertEqual(self.FLAVOR_PROFILE_ID, test_flavor.flavor_profile_id)

    def test_flavor_list(self):
        names = [fv.name for fv in self.conn.load_balancer.flavors()]
        self.assertIn(self.FLAVOR_NAME, names)

    def test_flavor_update(self):
        self.conn.load_balancer.update_flavor(
            self.FLAVOR_ID, name=self.UPDATE_NAME)
        test_flavor = self.conn.load_balancer.get_flavor(self.FLAVOR_ID)
        self.assertEqual(self.UPDATE_NAME, test_flavor.name)

        self.conn.load_balancer.update_flavor(
            self.FLAVOR_ID, name=self.FLAVOR_NAME)
        test_flavor = self.conn.load_balancer.get_flavor(self.FLAVOR_ID)
        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)

    def test_amphora_list(self):
        amp_ids = [amp.id for amp in self.conn.load_balancer.amphorae()]
        self.assertIn(self.AMPHORA_ID, amp_ids)

    def test_amphora_find(self):
        test_amphora = self.conn.load_balancer.find_amphora(self.AMPHORA_ID)
        self.assertEqual(self.AMPHORA_ID, test_amphora.id)

    def test_amphora_get(self):
        test_amphora = self.conn.load_balancer.get_amphora(self.AMPHORA_ID)
        self.assertEqual(self.AMPHORA_ID, test_amphora.id)

    def test_amphora_configure(self):
        self.conn.load_balancer.configure_amphora(self.AMPHORA_ID)
        test_amp = self.conn.load_balancer.get_amphora(self.AMPHORA_ID)
        self.assertEqual(self.AMPHORA_ID, test_amp.id)

    def test_amphora_failover(self):
        self.conn.load_balancer.failover_amphora(self.AMPHORA_ID)
        test_amp = self.conn.load_balancer.get_amphora(self.AMPHORA_ID)
        self.assertEqual(self.AMPHORA_ID, test_amp.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import openstack.config

from keystoneauth1 import discover
from openstack import connection
from openstack.tests import base


#: Defines the OpenStack Client Config (OCC) cloud key in your OCC config
#: file, typically in $HOME/.config/openstack/clouds.yaml. That configuration
#: will determine where the functional tests will be run and what resource
#: defaults will be used to run the functional tests.
TEST_CLOUD_NAME = os.getenv('OS_CLOUD', 'devstack-admin')
TEST_CLOUD_REGION = openstack.config.get_cloud_region(cloud=TEST_CLOUD_NAME)


def _get_resource_value(resource_key, default):
    try:
        return TEST_CLOUD_REGION.config['functional'][resource_key]
    except KeyError:
        return default


def _disable_keep_alive(conn):
    sess = conn.config.get_session()
    sess.keep_alive = False


IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.4.0-x86_64-disk')
FLAVOR_NAME = _get_resource_value('flavor_name', 'm1.small')


class BaseFunctionalTest(base.TestCase):

    _wait_for_timeout_key = ''

    def setUp(self):
        super(BaseFunctionalTest, self).setUp()
        self.conn = connection.Connection(config=TEST_CLOUD_REGION)
        _disable_keep_alive(self.conn)

        self._demo_name = os.environ.get('OPENSTACKSDK_DEMO_CLOUD', 'devstack')
        self._op_name = os.environ.get(
            'OPENSTACKSDK_OPERATOR_CLOUD', 'devstack-admin')

        self.config = openstack.config.OpenStackConfig()
        self._set_user_cloud()
        self._set_operator_cloud()

        self.identity_version = \
            self.operator_cloud.config.get_api_version('identity')

        # Defines default timeout for wait_for methods used
        # in the functional tests
        self._wait_for_timeout = int(
            os.getenv(self._wait_for_timeout_key, os.getenv(
                'OPENSTACKSDK_FUNC_TEST_TIMEOUT', 300)))

    def _set_user_cloud(self, **kwargs):
        user_config = self.config.get_one(
            cloud=self._demo_name, **kwargs)
        self.user_cloud = connection.Connection(config=user_config)
        _disable_keep_alive(self.user_cloud)

    def _set_operator_cloud(self, **kwargs):
        operator_config = self.config.get_one(
            cloud=self._op_name, **kwargs)
        self.operator_cloud = connection.Connection(config=operator_config)
        _disable_keep_alive(self.operator_cloud)

    def pick_image(self):
        images = self.user_cloud.list_images()
        self.add_info_on_exception('images', images)

        image_name = os.environ.get('OPENSTACKSDK_IMAGE')
        if image_name:
            for image in images:
                if image.name == image_name:
                    return image
            self.assertFalse(
                "Cloud does not have {image}".format(image=image_name))

        for image in images:
            if image.name.startswith('cirros') and image.name.endswith('-uec'):
                return image
        for image in images:
            if (image.name.startswith('cirros')
                    and image.disk_format == 'qcow2'):
                return image
        for image in images:
            if image.name.lower().startswith('ubuntu'):
                return image
        for image in images:
            if image.name.lower().startswith('centos'):
                return image
        self.assertFalse('no sensible image available')

    def addEmptyCleanup(self, func, *args, **kwargs):
        def cleanup():
            result = func(*args, **kwargs)
            self.assertIsNone(result)
        self.addCleanup(cleanup)

    def require_service(self, service_type, min_microversion=None, **kwargs):
        """Method to check whether a service exists

        Usage:
        class TestMeter(base.BaseFunctionalTest):
            ...
            def setUp(self):
                super(TestMeter, self).setUp()
                self.require_service('metering')

        :returns: True if the service exists, otherwise False.
        """
        if not self.conn.has_service(service_type):
            self.skipTest('Service {service_type} not found in cloud'.format(
                service_type=service_type))

        if not min_microversion:
            return

        data = self.conn.session.get_endpoint_data(
            service_type=service_type, **kwargs)

        if not (data.min_microversion
                and data.max_microversion
                and discover.version_between(
                    data.min_microversion,
                    data.max_microversion,
                    min_microversion)):
            self.skipTest('Service {service_type} does not provide '
                          'microversion {ver}'.format(
                              service_type=service_type,
                              ver=min_microversion))


class KeystoneBaseFunctionalTest(BaseFunctionalTest):

    def setUp(self):
        super(KeystoneBaseFunctionalTest, self).setUp()

        use_keystone_v2 = os.environ.get('OPENSTACKSDK_USE_KEYSTONE_V2', False)
        if use_keystone_v2:
            # keystone v2 has special behavior for the admin
            # interface and some of the operations, so make a new cloud
            # object with interface set to admin.
            # We only do it for keystone tests on v2 because otherwise
            # the admin interface is not a thing that wants to actually
            # be used
            self._set_operator_cloud(interface='admin')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from openstack.compute.v2 import hypervisor

EXAMPLE = {
    "status": "enabled",
    "service": {
        "host": "fake-mini",
        "disabled_reason": None,
        "id": 6
    },
    "vcpus_used": 0,
    "hypervisor_type": "QEMU",
    "local_gb_used": 0,
    "vcpus": 8,
    "hypervisor_hostname": "fake-mini",
    "memory_mb_used": 512,
    "memory_mb": 7980,
    "current_workload": 0,
    "state": "up",
    "host_ip": "23.253.248.171",
    "cpu_info": "some cpu info",
    "running_vms": 0,
    "free_disk_gb": 157,
    "hypervisor_version": 2000000,
    "disk_available_least": 140,
    "local_gb": 157,
    "free_ram_mb": 7468,
    "id": 1
}


class TestHypervisor(base.TestCase):

    def test_basic(self):
        sot = hypervisor.Hypervisor()
        self.assertEqual('hypervisor', sot.resource_key)
        self.assertEqual('hypervisors', sot.resources_key)
        self.assertEqual('/os-hypervisors', sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual({'hypervisor_hostname_pattern':
                              'hypervisor_hostname_pattern',
                              'limit': 'limit',
                              'marker': 'marker',
                              'with_servers': 'with_servers'},
                             sot._query_mapping._mapping)

    def test_make_it(self):
        sot = hypervisor.Hypervisor(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['hypervisor_hostname'], sot.name)
        self.assertEqual(EXAMPLE['state'], sot.state)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['service'], sot.service_details)
        self.assertEqual(EXAMPLE['vcpus_used'], sot.vcpus_used)
        self.assertEqual(EXAMPLE['hypervisor_type'], sot.hypervisor_type)
        self.assertEqual(EXAMPLE['local_gb_used'], sot.local_disk_used)
        self.assertEqual(EXAMPLE['vcpus'], sot.vcpus)
        self.assertEqual(EXAMPLE['vcpus_used'], sot.vcpus_used)
        self.assertEqual(EXAMPLE['memory_mb_used'], sot.memory_used)
        self.assertEqual(EXAMPLE['memory_mb'], sot.memory_size)
        self.assertEqual(EXAMPLE['current_workload'], sot.current_workload)
        self.assertEqual(EXAMPLE['host_ip'], sot.host_ip)
        self.assertEqual(EXAMPLE['cpu_info'], sot.cpu_info)
        self.assertEqual(EXAMPLE['running_vms'], sot.running_vms)
        self.assertEqual(EXAMPLE['free_disk_gb'], sot.local_disk_free)
        self.assertEqual(EXAMPLE['hypervisor_version'], sot.hypervisor_version)
        self.assertEqual(EXAMPLE['disk_available_least'], sot.disk_available)
        self.assertEqual(EXAMPLE['local_gb'], sot.local_disk_size)
        self.assertEqual(EXAMPLE['free_ram_mb'], sot.memory_free)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import testtools

import openstack
import openstack.cloud
from openstack.tests.unit import base


class TestNetwork(base.TestCase):

    mock_new_network_rep = {
        'provider:physical_network': None,
        'ipv6_address_scope': None,
        'revision_number': 3,
        'port_security_enabled': True,
        'provider:network_type': 'local',
        'id': '881d1bb7-a663-44c0-8f9f-ee2765b74486',
        'router:external': False,
        'availability_zone_hints': [],
        'availability_zones': [],
        'provider:segmentation_id': None,
        'ipv4_address_scope': None,
        'shared': False,
        'project_id': '861808a93da0484ea1767967c4df8a23',
        'status': 'ACTIVE',
        'subnets': [],
        'description': '',
        'tags': [],
        'updated_at': '2017-04-22T19:22:53Z',
        'is_default': False,
        'qos_policy_id': None,
        'name': 'netname',
        'admin_state_up': True,
        'tenant_id': '861808a93da0484ea1767967c4df8a23',
        'created_at': '2017-04-22T19:22:53Z',
        'mtu': 0,
        'dns_domain': 'sample.openstack.org.'
    }

    network_availability_zone_extension = {
        "alias": "network_availability_zone",
        "updated": "2015-01-01T10:00:00-00:00",
        "description": "Availability zone support for router.",
        "links": [],
        "name": "Network Availability Zone"
    }

    enabled_neutron_extensions = [network_availability_zone_extension]

    def test_list_networks(self):
        net1 = {'id': '1', 'name': 'net1'}
        net2 = {'id': '2', 'name': 'net2'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [net1, net2]})
        ])
        nets = self.cloud.list_networks()
        self.assertEqual([net1, net2], nets)
        self.assert_calls()

    def test_list_networks_filtered(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json'],
                     qs_elements=["name=test"]),
                 json={'networks': []})
        ])
        self.cloud.list_networks(filters={'name': 'test'})
        self.assert_calls()

    def test_create_network(self):
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'network': self.mock_new_network_rep},
                 validate=dict(
                     json={'network': {
                         'admin_state_up': True,
                         'name': 'netname'}}))
        ])
        network = self.cloud.create_network("netname")
        self.assertEqual(self.mock_new_network_rep, network)
        self.assert_calls()

    def test_create_network_specific_tenant(self):
        project_id = "project_id_value"
        mock_new_network_rep = copy.copy(self.mock_new_network_rep)
        mock_new_network_rep['project_id'] = project_id
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'network': mock_new_network_rep},
                 validate=dict(
                     json={'network': {
                         'admin_state_up': True,
                         'name': 'netname',
                         'tenant_id': project_id}}))
        ])
        network = self.cloud.create_network("netname", project_id=project_id)
        self.assertEqual(mock_new_network_rep, network)
        self.assert_calls()

    def test_create_network_external(self):
        mock_new_network_rep = copy.copy(self.mock_new_network_rep)
        mock_new_network_rep['router:external'] = True
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'network': mock_new_network_rep},
                 validate=dict(
                     json={'network': {
                         'admin_state_up': True,
                         'name': 'netname',
                         'router:external': True}}))
        ])
        network = self.cloud.create_network("netname", external=True)
        self.assertEqual(mock_new_network_rep, network)
        self.assert_calls()

    def test_create_network_provider(self):
        provider_opts = {'physical_network': 'mynet',
                         'network_type': 'vlan',
                         'segmentation_id': 'vlan1'}
        new_network_provider_opts = {
            'provider:physical_network': 'mynet',
            'provider:network_type': 'vlan',
            'provider:segmentation_id': 'vlan1'
        }
        mock_new_network_rep = copy.copy(self.mock_new_network_rep)
        mock_new_network_rep.update(new_network_provider_opts)
        expected_send_params = {
            'admin_state_up': True,
            'name': 'netname'
        }
        expected_send_params.update(new_network_provider_opts)
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'network': mock_new_network_rep},
                 validate=dict(
                     json={'network': expected_send_params}))
        ])
        network = self.cloud.create_network("netname", provider=provider_opts)
        self.assertEqual(mock_new_network_rep, network)
        self.assert_calls()

    def test_create_network_with_availability_zone_hints(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'extensions.json']),
                 json={'extensions': self.enabled_neutron_extensions}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'network': self.mock_new_network_rep},
                 validate=dict(
                     json={'network': {
                         'admin_state_up': True,
                         'name': 'netname',
                         'availability_zone_hints': ['nova']}}))
        ])
        network = self.cloud.create_network("netname",
                                            availability_zone_hints=['nova'])
        self.assertEqual(self.mock_new_network_rep, network)
        self.assert_calls()

    def test_create_network_provider_ignored_value(self):
        provider_opts = {'physical_network': 'mynet',
                         'network_type': 'vlan',
                         'segmentation_id': 'vlan1',
                         'should_not_be_passed': 1}
        new_network_provider_opts = {
            'provider:physical_network': 'mynet',
            'provider:network_type': 'vlan',
            'provider:segmentation_id': 'vlan1'
        }
        mock_new_network_rep = copy.copy(self.mock_new_network_rep)
        mock_new_network_rep.update(new_network_provider_opts)
        expected_send_params = {
            'admin_state_up': True,
            'name': 'netname'
        }
        expected_send_params.update(new_network_provider_opts)
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'network': mock_new_network_rep},
                 validate=dict(
                     json={'network': expected_send_params}))
        ])
        network = self.cloud.create_network("netname", provider=provider_opts)
        self.assertEqual(mock_new_network_rep, network)
        self.assert_calls()

    def test_create_network_wrong_availability_zone_hints_type(self):
        azh_opts = "invalid"
        with testtools.ExpectedException(
            openstack.cloud.OpenStackCloudException,
            "Parameter 'availability_zone_hints' must be a list"
        ):
            self.cloud.create_network("netname",
                                      availability_zone_hints=azh_opts)

    def test_create_network_provider_wrong_type(self):
        provider_opts = "invalid"
        with testtools.ExpectedException(
            openstack.cloud.OpenStackCloudException,
            "Parameter 'provider' must be a dict"
        ):
            self.cloud.create_network("netname", provider=provider_opts)

    def test_create_network_port_security_disabled(self):
        port_security_state = False
        mock_new_network_rep = copy.copy(self.mock_new_network_rep)
        mock_new_network_rep['port_security_enabled'] = port_security_state
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'network': mock_new_network_rep},
                 validate=dict(
                     json={'network': {
                         'admin_state_up': True,
                         'name': 'netname',
                         'port_security_enabled': port_security_state}}))
        ])
        network = self.cloud.create_network(
            "netname",
            port_security_enabled=port_security_state
        )
        self.assertEqual(mock_new_network_rep, network)
        self.assert_calls()

    def test_create_network_with_mtu(self):
        mtu_size = 1500
        mock_new_network_rep = copy.copy(self.mock_new_network_rep)
        mock_new_network_rep['mtu'] = mtu_size
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'network': mock_new_network_rep},
                 validate=dict(
                     json={'network': {
                         'admin_state_up': True,
                         'name': 'netname',
                         'mtu': mtu_size}}))
        ])
        network = self.cloud.create_network("netname",
                                            mtu_size=mtu_size
                                            )
        self.assertEqual(mock_new_network_rep, network)
        self.assert_calls()

    def test_create_network_with_wrong_mtu_size(self):
        with testtools.ExpectedException(
                openstack.cloud.OpenStackCloudException,
                "Parameter 'mtu_size' must be greater than 67."
        ):
            self.cloud.create_network("netname", mtu_size=42)

    def test_create_network_with_wrong_mtu_type(self):
        with testtools.ExpectedException(
                openstack.cloud.OpenStackCloudException,
                "Parameter 'mtu_size' must be an integer."
        ):
            self.cloud.create_network("netname", mtu_size="fourty_two")

    def test_delete_network(self):
        network_id = "test-net-id"
        network_name = "network"
        network = {'id': network_id, 'name': network_name}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [network]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'networks', "%s.json" % network_id]),
                 json={})
        ])
        self.assertTrue(self.cloud.delete_network(network_name))
        self.assert_calls()

    def test_delete_network_not_found(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': []}),
        ])
        self.assertFalse(self.cloud.delete_network('test-net'))
        self.assert_calls()

    def test_delete_network_exception(self):
        network_id = "test-net-id"
        network_name = "network"
        network = {'id': network_id, 'name': network_name}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [network]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'networks', "%s.json" % network_id]),
                 status_code=503)
        ])
        self.assertRaises(openstack.cloud.OpenStackCloudException,
                          self.cloud.delete_network, network_name)
        self.assert_calls()

    def test_get_network_by_id(self):
        network_id = "test-net-id"
        network_name = "network"
        network = {'id': network_id, 'name': network_name}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'networks', "%s" % network_id]),
                 json={'network': network})
        ])
        self.assertTrue(self.cloud.get_network_by_id(network_id))
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Delete resources with the Network service.

For a full guide see
https://docs.openstack.org/openstacksdk/latest/user/guides/network.html
"""


def delete_network(conn):
    print("Delete Network:")

    example_network = conn.network.find_network(
        'openstacksdk-example-project-network')

    for example_subnet in example_network.subnet_ids:
        conn.network.delete_subnet(example_subnet, ignore_missing=False)
    conn.network.delete_network(example_network, ignore_missing=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import random

from openstack import connection
from openstack.tests.functional import base


class TestZone(base.BaseFunctionalTest):

    def setUp(self):
        super(TestZone, self).setUp()
        self.require_service('dns')

        self.conn = connection.from_config(cloud_name=base.TEST_CLOUD_NAME)

        # Note: zone deletion is not an immediate operation, so each time
        # chose a new zone name for a test
        # getUniqueString is not guaranteed to return unique string between
        # different tests of the same class.
        self.ZONE_NAME = 'example-{0}.org.'.format(random.randint(1, 10000))

        self.zone = self.conn.dns.create_zone(
            name=self.ZONE_NAME,
            email='joe@example.org',
            type='PRIMARY',
            ttl=7200,
            description='example zone'
        )
        self.addCleanup(self.conn.dns.delete_zone, self.zone)

    def test_get_zone(self):
        zone = self.conn.dns.get_zone(self.zone)
        self.assertEqual(self.zone, zone)

    def test_list_zones(self):
        names = [f.name for f in self.conn.dns.zones()]
        self.assertIn(self.ZONE_NAME, names)

    def test_create_rs(self):
        zone = self.conn.dns.get_zone(self.zone)
        self.assertIsNotNone(self.conn.dns.create_recordset(
            zone=zone,
            name='www.{zone}'.format(zone=zone.name),
            type='A',
            description='Example zone rec',
            ttl=3600,
            records=['192.168.1.1']
        ))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.baremetal.v1 import _common
from openstack import exceptions
from openstack import resource
from openstack import utils


class ValidationResult(object):
    """Result of a single interface validation.

    :ivar result: Result of a validation, ``True`` for success, ``False`` for
        failure, ``None`` for unsupported interface.
    :ivar reason: If ``result`` is ``False`` or ``None``, explanation of
        the result.
    """

    def __init__(self, result, reason):
        self.result = result
        self.reason = reason


class Node(_common.ListMixin, resource.Resource):

    resources_key = 'nodes'
    base_path = '/nodes'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    allow_patch = True
    commit_method = 'PATCH'
    commit_jsonpatch = True

    _query_mapping = resource.QueryParameters(
        'associated', 'conductor_group', 'driver', 'fault',
        'provision_state', 'resource_class',
        fields={'type': _common.fields_type},
        instance_id='instance_uuid',
        is_maintenance='maintenance',
    )

    # The allocation_uuid field introduced in 1.52 (Stein).
    _max_microversion = '1.52'

    # Properties
    #: The UUID of the allocation associated with this node. Added in API
    #: microversion 1.52.
    allocation_id = resource.Body("allocation_uuid")
    #: The UUID of the chassis associated wit this node. Can be empty or None.
    chassis_id = resource.Body("chassis_uuid")
    #: The current clean step.
    clean_step = resource.Body("clean_step")
    #: Hostname of the conductor currently handling this ndoe. Added in API
    # microversion 1.49.
    conductor = resource.Body("conductor")
    #: Conductor group this node is managed by. Added in API microversion 1.46.
    conductor_group = resource.Body("conductor_group")
    #: Timestamp at which the node was last updated.
    created_at = resource.Body("created_at")
    #: The current deploy step. Added in API microversion 1.44.
    deploy_step = resource.Body("deploy_step")
    #: The name of the driver.
    driver = resource.Body("driver")
    #: All the metadata required by the driver to manage this node. List of
    #: fields varies between drivers, and can be retrieved from the
    #: :class:`openstack.baremetal.v1.driver.Driver` resource.
    driver_info = resource.Body("driver_info", type=dict)
    #: Internal metadata set and stored by node's driver. This is read-only.
    driver_internal_info = resource.Body("driver_internal_info", type=dict)
    #: A set of one or more arbitrary metadata key and value pairs.
    extra = resource.Body("extra")
    #: Fault type that caused the node to enter maintenance mode.
    #: Introduced in API microversion 1.42.
    fault = resource.Body("fault")
    #: The UUID of the node resource.
    id = resource.Body("uuid", alternate_id=True)
    #: Information used to customize the deployed image, e.g. size of root
    #: partition, config drive in the form of base64 encoded string and other
    #: metadata.
    instance_info = resource.Body("instance_info")
    #: UUID of the nova instance associated with this node.
    instance_id = resource.Body("instance_uuid")
    #: Override enabling of automated cleaning. Added in API microversion 1.47.
    is_automated_clean_enabled = resource.Body("automated_clean", type=bool)
    #: Whether console access is enabled on this node.
    is_console_enabled = resource.Body("console_enabled", type=bool)
    #: Whether node is currently in "maintenance mode". Nodes put into
    #: maintenance mode are removed from the available resource pool.
    is_maintenance = resource.Body("maintenance", type=bool)
    # Whether the node is protected from undeploying. Added in API microversion
    # 1.48.
    is_protected = resource.Body("protected", type=bool)
    #: Any error from the most recent transaction that started but failed to
    #: finish.
    last_error = resource.Body("last_error")
    #: A list of relative links, including self and bookmark links.
    links = resource.Body("links", type=list)
    #: user settable description of the reason why the node was placed into
    #: maintenance mode.
    maintenance_reason = resource.Body("maintenance_reason")
    #: Human readable identifier for the node. May be undefined. Certain words
    #: are reserved. Added in API microversion 1.5
    name = resource.Body("name")
    #: Links to the collection of ports on this node.
    ports = resource.Body("ports", type=list)
    #: Links to the collection of portgroups on this node. Available since
    #: API microversion 1.24.
    port_groups = resource.Body("portgroups", type=list)
    #: The current power state. Usually "power on" or "power off", but may be
    #: "None" if service is unable to determine the power state.
    power_state = resource.Body("power_state")
    #: Physical characteristics of the node. Content populated by the service
    #: during inspection.
    properties = resource.Body("properties", type=dict)
    # The reason why this node is protected. Added in API microversion 1.48.
    protected_reason = resource.Body("protected_reason")
    #: The current provisioning state of the node.
    provision_state = resource.Body("provision_state")
    #: The current RAID configuration of the node.
    raid_config = resource.Body("raid_config")
    #: The name of an service conductor host which is holding a lock on this
    #: node, if a lock is held.
    reservation = resource.Body("reservation")
    #: A string to be used by external schedulers to identify this node as a
    #: unit of a specific type of resource. Added in API microversion 1.21.
    resource_class = resource.Body("resource_class")
    #: Links to the collection of states.
    states = resource.Body("states", type=list)
    #: The requested state if a provisioning action has been requested. For
    #: example, ``AVAILABLE``, ``DEPLOYING``, ``DEPLOYWAIT``, ``DEPLOYING``,
    #: ``ACTIVE`` etc.
    target_provision_state = resource.Body("target_provision_state")
    #: The requested state during a state transition.
    target_power_state = resource.Body("target_power_state")
    #: The requested RAID configuration of the node which will be applied when
    #: the node next transitions through the CLEANING state.
    target_raid_config = resource.Body("target_raid_config")
    #: Traits of the node. Introduced in API microversion 1.37.
    traits = resource.Body("traits", type=list)
    #: Timestamp at which the node was last updated.
    updated_at = resource.Body("updated_at")

    # Hardware interfaces grouped together for convenience.

    #: BIOS interface to use when setting BIOS properties of the node.
    #: Introduced in API microversion 1.40.
    bios_interface = resource.Body("bios_interface")
    #: Boot interface to use when configuring boot of the node.
    #: Introduced in API microversion 1.31.
    boot_interface = resource.Body("boot_interface")
    #: Console interface to use when working with serial console.
    #: Introduced in API microversion 1.31.
    console_interface = resource.Body("console_interface")
    #: Deploy interface to use when deploying the node.
    #: Introduced in API microversion 1.31.
    deploy_interface = resource.Body("deploy_interface")
    #: Inspect interface to use when inspecting the node.
    #: Introduced in API microversion 1.31.
    inspect_interface = resource.Body("inspect_interface")
    #: Management interface to use for management actions on the node.
    #: Introduced in API microversion 1.31.
    management_interface = resource.Body("management_interface")
    #: Network interface provider to use when plumbing the network connections
    #: for this node. Introduced in API microversion 1.20.
    network_interface = resource.Body("network_interface")
    #: Power interface to use for power actions on the node.
    #: Introduced in API microversion 1.31.
    power_interface = resource.Body("power_interface")
    #: RAID interface to use for configuring RAID on the node.
    #: Introduced in API microversion 1.31.
    raid_interface = resource.Body("raid_interface")
    #: Rescue interface to use for rescuing of the node.
    #: Introduced in API microversion 1.38.
    rescue_interface = resource.Body("rescue_interface")
    #: Storage interface to use when attaching remote storage.
    #: Introduced in API microversion 1.33.
    storage_interface = resource.Body("storage_interface")
    #: Vendor interface to use for vendor-specific actions on the node.
    #: Introduced in API microversion 1.31.
    vendor_interface = resource.Body("vendor_interface")

    def _consume_body_attrs(self, attrs):
        if 'provision_state' in attrs and attrs['provision_state'] is None:
            # API version 1.1 uses None instead of "available". Make it
            # consistent.
            attrs['provision_state'] = 'available'
        return super(Node, self)._consume_body_attrs(attrs)

    def create(self, session, *args, **kwargs):
        """Create a remote resource based on this instance.

        The overridden version is capable of handling the populated
        ``provision_state`` field of one of three values: ``enroll``,
        ``manageable`` or ``available``. The default is currently
        ``available``, since it's the only state supported by all API versions.

        Note that Bare Metal API 1.4 is required for ``manageable`` and
        1.11 is required for ``enroll``.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :return: This :class:`Resource` instance.
        :raises: ValueError if the Node's ``provision_state`` is not one of
            ``None``, ``enroll``, ``manageable`` or ``available``.
        :raises: :exc:`~openstack.exceptions.NotSupported` if
            the ``provision_state`` cannot be reached with any API version
            supported by the server.
        """
        expected_provision_state = self.provision_state
        if expected_provision_state is None:
            expected_provision_state = 'available'

        if expected_provision_state not in ('enroll',
                                            'manageable',
                                            'available'):
            raise ValueError(
                "Node's provision_state must be one of 'enroll', "
                "'manageable' or 'available' for creation, got %s" %
                expected_provision_state)

        session = self._get_session(session)
        # Verify that the requested provision state is reachable with the API
        # version we are going to use.
        try:
            expected_version = _common.STATE_VERSIONS[expected_provision_state]
        except KeyError:
            pass
        else:
            self._assert_microversion_for(
                session, 'create', expected_version,
                error_message="Cannot create a node with initial provision "
                "state %s" % expected_provision_state)

        # Ironic cannot set provision_state itself, so marking it as unchanged
        self._clean_body_attrs({'provision_state'})
        super(Node, self).create(session, *args, **kwargs)

        if (self.provision_state == 'enroll'
                and expected_provision_state != 'enroll'):
            self.set_provision_state(session, 'manage', wait=True)

        if (self.provision_state == 'manageable'
                and expected_provision_state == 'available'):
            self.set_provision_state(session, 'provide', wait=True)

        if (self.provision_state == 'available'
                and expected_provision_state == 'manageable'):
            self.set_provision_state(session, 'manage', wait=True)

        return self

    def commit(self, session, *args, **kwargs):
        """Commit the state of the instance to the remote resource.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :return: This :class:`Node` instance.
        """
        # These fields have to be set through separate API.
        if ('maintenance_reason' in self._body.dirty
                or 'maintenance' in self._body.dirty):
            if not self.is_maintenance and self.maintenance_reason:
                if 'maintenance' in self._body.dirty:
                    self.maintenance_reason = None
                else:
                    raise ValueError('Maintenance reason cannot be set when '
                                     'maintenance is False')
            if self.is_maintenance:
                self._do_maintenance_action(
                    session, 'put', {'reason': self.maintenance_reason})
            else:
                # This corresponds to setting maintenance=False and
                # maintenance_reason=None in the same request.
                self._do_maintenance_action(session, 'delete')

            self._clean_body_attrs({'maintenance', 'maintenance_reason'})
            if not self.requires_commit:
                # Other fields are not updated, re-fetch the node to reflect
                # the new status.
                return self.fetch(session)

        return super(Node, self).commit(session, *args, **kwargs)

    def set_provision_state(self, session, target, config_drive=None,
                            clean_steps=None, rescue_password=None,
                            wait=False, timeout=None):
        """Run an action modifying this node's provision state.

        This call is asynchronous, it will return success as soon as the Bare
        Metal service acknowledges the request.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param target: Provisioning action, e.g. ``active``, ``provide``.
            See the Bare Metal service documentation for available actions.
        :param config_drive: Config drive to pass to the node, only valid
            for ``active` and ``rebuild`` targets. You can use functions from
            :mod:`openstack.baremetal.configdrive` to build it.
        :param clean_steps: Clean steps to execute, only valid for ``clean``
            target.
        :param rescue_password: Password for the rescue operation, only valid
            for ``rescue`` target.
        :param wait: Whether to wait for the target state to be reached.
        :param timeout: Timeout (in seconds) to wait for the target state to be
            reached. If ``None``, wait without timeout.

        :return: This :class:`Node` instance.
        :raises: ValueError if ``config_drive``, ``clean_steps`` or
            ``rescue_password`` are provided with an invalid ``target``.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the node
            reaches an error state while waiting for the state.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if timeout
            is reached while waiting for the state.
        """
        session = self._get_session(session)

        version = None
        if target in _common.PROVISIONING_VERSIONS:
            version = '1.%d' % _common.PROVISIONING_VERSIONS[target]

        if config_drive:
            # Some config drive actions require a higher version.
            if isinstance(config_drive, dict):
                version = _common.CONFIG_DRIVE_DICT_VERSION
            elif target == 'rebuild':
                version = _common.CONFIG_DRIVE_REBUILD_VERSION

        version = self._assert_microversion_for(session, 'commit', version)

        body = {'target': target}
        if config_drive:
            if target not in ('active', 'rebuild'):
                raise ValueError('Config drive can only be provided with '
                                 '"active" and "rebuild" targets')
            # Not a typo - ironic accepts "configdrive" (without underscore)
            body['configdrive'] = config_drive

        if clean_steps is not None:
            if target != 'clean':
                raise ValueError('Clean steps can only be provided with '
                                 '"clean" target')
            body['clean_steps'] = clean_steps

        if rescue_password is not None:
            if target != 'rescue':
                raise ValueError('Rescue password can only be provided with '
                                 '"rescue" target')
            body['rescue_password'] = rescue_password

        if wait:
            try:
                expected_state = _common.EXPECTED_STATES[target]
            except KeyError:
                raise ValueError('For target %s the expected state is not '
                                 'known, cannot wait for it' % target)

        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'states', 'provision')
        response = session.put(
            request.url, json=body,
            headers=request.headers, microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

        msg = ("Failed to set provision state for bare metal node {node} "
               "to {target}".format(node=self.id, target=target))
        exceptions.raise_from_response(response, error_message=msg)

        if wait:
            return self.wait_for_provision_state(session,
                                                 expected_state,
                                                 timeout=timeout)
        else:
            return self.fetch(session)

    def wait_for_provision_state(self, session, expected_state, timeout=None,
                                 abort_on_failed_state=True):
        """Wait for the node to reach the expected state.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param expected_state: The expected provisioning state to reach.
        :param timeout: If ``wait`` is set to ``True``, specifies how much (in
            seconds) to wait for the expected state to be reached. The value of
            ``None`` (the default) means no client-side timeout.
        :param abort_on_failed_state: If ``True`` (the default), abort waiting
            if the node reaches a failure state which does not match the
            expected one. Note that the failure state for ``enroll`` ->
            ``manageable`` transition is ``enroll`` again.

        :return: This :class:`Node` instance.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the node
            reaches an error state and ``abort_on_failed_state`` is ``True``.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` on timeout.
        """
        for count in utils.iterate_timeout(
                timeout,
                "Timeout waiting for node %(node)s to reach "
                "target state '%(state)s'" % {'node': self.id,
                                              'state': expected_state}):
            self.fetch(session)
            if self._check_state_reached(session, expected_state,
                                         abort_on_failed_state):
                return self

            session.log.debug(
                'Still waiting for node %(node)s to reach state '
                '"%(target)s", the current state is "%(state)s"',
                {'node': self.id, 'target': expected_state,
                 'state': self.provision_state})

    def wait_for_reservation(self, session, timeout=None):
        """Wait for a lock on the node to be released.

        Bare metal nodes in ironic have a reservation lock that
        is used to represent that a conductor has locked the node
        while performing some sort of action, such as changing
        configuration as a result of a machine state change.

        This lock can occur during power syncronization, and prevents
        updates to objects attached to the node, such as ports.

        Note that nothing prevents a conductor from acquiring the lock again
        after this call returns, so it should be treated as best effort.

        Returns immediately if there is no reservation on the node.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param timeout: How much (in seconds) to wait for the lock to be
            released. The value of ``None`` (the default) means no timeout.

        :return: This :class:`Node` instance.
        """
        if self.reservation is None:
            return self

        for count in utils.iterate_timeout(
                timeout,
                "Timeout waiting for the lock to be released on node %s" %
                self.id):
            self.fetch(session)
            if self.reservation is None:
                return self

            session.log.debug(
                'Still waiting for the lock to be released on node '
                '%(node)s, currently locked by conductor %(host)s',
                {'node': self.id, 'host': self.reservation})

    def _check_state_reached(self, session, expected_state,
                             abort_on_failed_state=True):
        """Wait for the node to reach the expected state.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param expected_state: The expected provisioning state to reach.
        :param abort_on_failed_state: If ``True`` (the default), abort waiting
            if the node reaches a failure state which does not match the
            expected one. Note that the failure state for ``enroll`` ->
            ``manageable`` transition is ``enroll`` again.

        :return: ``True`` if the target state is reached
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the node
            reaches an error state and ``abort_on_failed_state`` is ``True``.
        """
        # NOTE(dtantsur): microversion 1.2 changed None to available
        if (self.provision_state == expected_state
                or (expected_state == 'available'
                    and self.provision_state is None)):
            return True
        elif not abort_on_failed_state:
            return False

        if (self.provision_state.endswith(' failed') or
                self.provision_state == 'error'):
            raise exceptions.ResourceFailure(
                "Node %(node)s reached failure state \"%(state)s\"; "
                "the last error is %(error)s" %
                {'node': self.id, 'state': self.provision_state,
                 'error': self.last_error})
        # Special case: a failure state for "manage" transition can be
        # "enroll"
        elif (expected_state == 'manageable'
              and self.provision_state == 'enroll' and self.last_error):
            raise exceptions.ResourceFailure(
                "Node %(node)s could not reach state manageable: "
                "failed to verify management credentials; "
                "the last error is %(error)s" %
                {'node': self.id, 'error': self.last_error})

    # TODO(dtantsur): waiting for power state
    def set_power_state(self, session, target):
        """Run an action modifying this node's power state.

        This call is asynchronous, it will return success as soon as the Bare
        Metal service acknowledges the request.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param target: Target power state, e.g. "rebooting", "power on".
            See the Bare Metal service documentation for available actions.
        """
        session = self._get_session(session)

        if target.startswith("soft "):
            version = '1.27'
        else:
            version = None

        version = self._assert_microversion_for(session, 'commit', version)

        # TODO(dtantsur): server timeout support
        body = {'target': target}

        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'states', 'power')
        response = session.put(
            request.url, json=body,
            headers=request.headers, microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

        msg = ("Failed to set power state for bare metal node {node} "
               "to {target}".format(node=self.id, target=target))
        exceptions.raise_from_response(response, error_message=msg)

    def attach_vif(self, session, vif_id, retry_on_conflict=True):
        """Attach a VIF to the node.

        The exact form of the VIF ID depends on the network interface used by
        the node. In the most common case it is a Network service port
        (NOT a Bare Metal port) ID. A VIF can only be attached to one node
        at a time.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param string vif_id: Backend-specific VIF ID.
        :param retry_on_conflict: Whether to retry HTTP CONFLICT errors.
            This can happen when either the VIF is already used on a node or
            the node is locked. Since the latter happens more often, the
            default value is True.
        :return: ``None``
        :raises: :exc:`~openstack.exceptions.NotSupported` if the server
            does not support the VIF API.
        """
        session = self._get_session(session)
        version = self._assert_microversion_for(
            session, 'commit', _common.VIF_VERSION,
            error_message=("Cannot use VIF attachment API"))

        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'vifs')
        body = {'id': vif_id}
        retriable_status_codes = _common.RETRIABLE_STATUS_CODES
        if not retry_on_conflict:
            retriable_status_codes = set(retriable_status_codes) - {409}
        response = session.post(
            request.url, json=body,
            headers=request.headers, microversion=version,
            retriable_status_codes=retriable_status_codes)

        msg = ("Failed to attach VIF {vif} to bare metal node {node}"
               .format(node=self.id, vif=vif_id))
        exceptions.raise_from_response(response, error_message=msg)

    def detach_vif(self, session, vif_id, ignore_missing=True):
        """Detach a VIF from the node.

        The exact form of the VIF ID depends on the network interface used by
        the node. In the most common case it is a Network service port
        (NOT a Bare Metal port) ID.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param string vif_id: Backend-specific VIF ID.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the VIF does not exist. Otherwise, ``False``
                    is returned.
        :return: ``True`` if the VIF was detached, otherwise ``False``.
        :raises: :exc:`~openstack.exceptions.NotSupported` if the server
            does not support the VIF API.
        """
        session = self._get_session(session)
        version = self._assert_microversion_for(
            session, 'commit', _common.VIF_VERSION,
            error_message=("Cannot use VIF attachment API"))

        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'vifs', vif_id)
        response = session.delete(
            request.url, headers=request.headers, microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

        if ignore_missing and response.status_code == 400:
            session.log.debug(
                'VIF %(vif)s was already removed from node %(node)s',
                {'vif': vif_id, 'node': self.id})
            return False

        msg = ("Failed to detach VIF {vif} from bare metal node {node}"
               .format(node=self.id, vif=vif_id))
        exceptions.raise_from_response(response, error_message=msg)
        return True

    def list_vifs(self, session):
        """List IDs of VIFs attached to the node.

        The exact form of the VIF ID depends on the network interface used by
        the node. In the most common case it is a Network service port
        (NOT a Bare Metal port) ID.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :return: List of VIF IDs as strings.
        :raises: :exc:`~openstack.exceptions.NotSupported` if the server
            does not support the VIF API.
        """
        session = self._get_session(session)
        version = self._assert_microversion_for(
            session, 'fetch', _common.VIF_VERSION,
            error_message=("Cannot use VIF attachment API"))

        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'vifs')
        response = session.get(
            request.url, headers=request.headers, microversion=version)

        msg = ("Failed to list VIFs attached to bare metal node {node}"
               .format(node=self.id))
        exceptions.raise_from_response(response, error_message=msg)
        return [vif['id'] for vif in response.json()['vifs']]

    def validate(self, session, required=('boot', 'deploy', 'power')):
        """Validate required information on a node.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param required: List of interfaces that are required to pass
            validation. The default value is the list of minimum required
            interfaces for provisioning.

        :return: dict mapping interface names to :class:`ValidationResult`
            objects.
        :raises: :exc:`~openstack.exceptions.ValidationException` if validation
            fails for a required interface.
        """
        session = self._get_session(session)
        version = self._get_microversion_for(session, 'fetch')

        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'validate')
        response = session.get(request.url, headers=request.headers,
                               microversion=version)

        msg = ("Failed to validate node {node}".format(node=self.id))
        exceptions.raise_from_response(response, error_message=msg)
        result = response.json()

        if required:
            failed = [
                '%s (%s)' % (key, value.get('reason', 'no reason'))
                for key, value in result.items()
                if key in required and not value.get('result')
            ]

            if failed:
                raise exceptions.ValidationException(
                    'Validation failed for required interfaces of node {node}:'
                    ' {failures}'.format(node=self.id,
                                         failures=', '.join(failed)))

        return {key: ValidationResult(value.get('result'), value.get('reason'))
                for key, value in result.items()}

    def set_maintenance(self, session, reason=None):
        """Enable maintenance mode on the node.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param reason: Optional reason for maintenance.
        :return: This :class:`Node` instance.
        """
        self._do_maintenance_action(session, 'put', {'reason': reason})
        return self.fetch(session)

    def unset_maintenance(self, session):
        """Disable maintenance mode on the node.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :return: This :class:`Node` instance.
        """
        self._do_maintenance_action(session, 'delete')
        return self.fetch(session)

    def _do_maintenance_action(self, session, verb, body=None):
        session = self._get_session(session)
        version = self._get_microversion_for(session, 'commit')
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'maintenance')
        response = getattr(session, verb)(
            request.url, json=body,
            headers=request.headers, microversion=version)
        msg = ("Failed to change maintenance mode for node {node}"
               .format(node=self.id))
        exceptions.raise_from_response(response, error_message=msg)

    def set_boot_device(self, session, boot_device, persistent=False):
        """Set node boot device

        :param session: The session to use for making this request.
        :param boot_device: Boot device to assign to the node.
        :param persistent: If the boot device change is maintained after node
            reboot
        :return: The updated :class:`~openstack.baremetal.v1.node.Node`
        """
        session = self._get_session(session)
        version = self._get_microversion_for(session, 'commit')
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'management', 'boot_device')

        body = {'boot_device': boot_device, 'persistent': persistent}

        response = session.put(
            request.url, json=body,
            headers=request.headers, microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

        msg = ("Failed to set boot device for node {node}"
               .format(node=self.id))
        exceptions.raise_from_response(response, error_message=msg)

    def add_trait(self, session, trait):
        """Add a trait to a node.

        :param session: The session to use for making this request.
        :param trait: The trait to add to the node.
        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
        """
        session = self._get_session(session)
        version = utils.pick_microversion(session, '1.37')
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'traits', trait)
        response = session.put(
            request.url, json=None,
            headers=request.headers, microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

        msg = ("Failed to add trait {trait} for node {node}"
               .format(trait=trait, node=self.id))
        exceptions.raise_from_response(response, error_message=msg)

        self.traits = list(set(self.traits or ()) | {trait})

    def remove_trait(self, session, trait, ignore_missing=True):
        """Remove a trait from a node.

        :param session: The session to use for making this request.
        :param trait: The trait to remove from the node.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the trait does not exist.
            Otherwise, ``False`` is returned.
        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
        """
        session = self._get_session(session)
        version = utils.pick_microversion(session, '1.37')
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'traits', trait)

        response = session.delete(
            request.url, headers=request.headers, microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

        if ignore_missing or response.status_code == 400:
            session.log.debug(
                'Trait %(trait)s was already removed from node %(node)s',
                {'trait': trait, 'node': self.id})
            return False

        msg = ("Failed to remove trait {trait} from bare metal node {node}"
               .format(node=self.id, trait=trait))
        exceptions.raise_from_response(response, error_message=msg)

        self.traits = list(set(self.traits) - {trait})

        return True

    def set_traits(self, session, traits):
        """Set traits for a node.

        Removes any existing traits and adds the traits passed in to this
        method.

        :param session: The session to use for making this request.
        :param traits: list of traits to add to the node.
        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
        """
        session = self._get_session(session)
        version = utils.pick_microversion(session, '1.37')
        request = self._prepare_request(requires_id=True)
        request.url = utils.urljoin(request.url, 'traits')

        body = {'traits': traits}

        response = session.put(
            request.url, json=body,
            headers=request.headers, microversion=version,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)

        msg = ("Failed to set traits for node {node}"
               .format(node=self.id))
        exceptions.raise_from_response(response, error_message=msg)

        self.traits = traits


NodeDetail = Node



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack import resource


class Claim(resource.Resource):
    # FIXME(anyone): The name string of `location` field of Zaqar API response
    # is lower case. That is inconsistent with the guide from API-WG. This is
    # a workaround for this issue.
    location = resource.Header("location")

    resources_key = 'claims'
    base_path = '/queues/%(queue_name)s/claims'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    commit_method = 'PATCH'

    # Properties
    #: The value in seconds indicating how long the claim has existed.
    age = resource.Body("age")
    #: In case worker stops responding for a long time, the server will
    #: extend the lifetime of claimed messages to be at least as long as
    #: the lifetime of the claim itself, plus the specified grace period.
    #: Must between 60 and 43200 seconds(12 hours).
    grace = resource.Body("grace")
    #: The number of messages to claim. Default 10, up to 20.
    limit = resource.Body("limit")
    #: Messages have been successfully claimed.
    messages = resource.Body("messages")
    #: Number of seconds the server wait before releasing the claim. Must
    #: between 60 and 43200 seconds(12 hours).
    ttl = resource.Body("ttl")
    #: The name of queue to claim message from.
    queue_name = resource.URI("queue_name")
    #: The ID to identify the client accessing Zaqar API. Must be specified
    #: in header for each API request.
    client_id = resource.Header("Client-ID")
    #: The ID to identify the project. Must be provided when keystone
    #: authentication is not enabled in Zaqar service.
    project_id = resource.Header("X-PROJECT-ID")

    def _translate_response(self, response, has_body=True):
        super(Claim, self)._translate_response(response, has_body=has_body)
        if has_body and self.location:
            # Extract claim ID from location
            self.id = self.location.split("claims/")[1]

    def create(self, session, prepend_key=False, base_path=None):
        request = self._prepare_request(requires_id=False,
                                        prepend_key=prepend_key,
                                        base_path=base_path)
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id()
        }
        request.headers.update(headers)
        response = session.post(request.url,
                                json=request.body, headers=request.headers)

        # For case no message was claimed successfully, 204 No Content
        # message will be returned. In other cases, we translate response
        # body which has `messages` field(list) included.
        if response.status_code != 204:
            self._translate_response(response)

        return self

    def fetch(self, session, requires_id=True,
              base_path=None, error_message=None):
        request = self._prepare_request(requires_id=requires_id,
                                        base_path=base_path)
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id()
        }

        request.headers.update(headers)
        response = session.get(request.url,
                               headers=request.headers)
        self._translate_response(response)

        return self

    def commit(self, session, prepend_key=False, has_body=False,
               base_path=None):
        request = self._prepare_request(prepend_key=prepend_key,
                                        base_path=base_path)
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id()
        }

        request.headers.update(headers)
        session.patch(request.url,
                      json=request.body, headers=request.headers)

        return self

    def delete(self, session):
        request = self._prepare_request()
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id()
        }

        request.headers.update(headers)
        response = session.delete(request.url,
                                  headers=request.headers)

        self._translate_response(response, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import network
from openstack.network.v2 import rbac_policy
from openstack.tests.functional import base


class TestRBACPolicy(base.BaseFunctionalTest):

    ACTION = 'access_as_shared'
    OBJ_TYPE = 'network'
    TARGET_TENANT_ID = '*'
    NET_ID = None
    ID = None

    def setUp(self):
        super(TestRBACPolicy, self).setUp()
        self.NET_NAME = self.getUniqueString('net')
        self.UPDATE_NAME = self.getUniqueString()
        net = self.conn.network.create_network(name=self.NET_NAME)
        assert isinstance(net, network.Network)
        self.NET_ID = net.id

        sot = self.conn.network.create_rbac_policy(
            action=self.ACTION,
            object_type=self.OBJ_TYPE,
            target_tenant=self.TARGET_TENANT_ID,
            object_id=self.NET_ID)
        assert isinstance(sot, rbac_policy.RBACPolicy)
        self.ID = sot.id

    def tearDown(self):
        sot = self.conn.network.delete_rbac_policy(
            self.ID,
            ignore_missing=False)
        self.assertIsNone(sot)
        sot = self.conn.network.delete_network(
            self.NET_ID,
            ignore_missing=False)
        self.assertIsNone(sot)
        super(TestRBACPolicy, self).tearDown()

    def test_find(self):
        sot = self.conn.network.find_rbac_policy(self.ID)
        self.assertEqual(self.ID, sot.id)

    def test_get(self):
        sot = self.conn.network.get_rbac_policy(self.ID)
        self.assertEqual(self.ID, sot.id)

    def test_list(self):
        ids = [o.id for o in self.conn.network.rbac_policies()]
        self.assertIn(self.ID, ids)
